from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2010 - 2012 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

import sys

from subscription_manager.cli import AbstractCLICommand


class RCTCliCommand(AbstractCLICommand):
    FILE_ARG_IDX = 0

    def __init__(self, name="cli", aliases=None, shortdesc=None, primary=False):
        AbstractCLICommand.__init__(self, name=name, aliases=aliases,
                shortdesc=shortdesc, primary=primary)

    def main(self, args=None):
        # In testing we sometimes specify args, otherwise use the default:
        if args is None:
            # Skip the program name and the command name.
            args = sys.argv[2:]

        (self.options, self.args) = self.parser.parse_args(args)

        self._validate_options()
        return_code = self._do_command()
        if return_code is not None:
            return return_code

    def _get_file_from_args(self):
        if not len(self.args) > self.FILE_ARG_IDX:
            return ''
        return self.args[self.FILE_ARG_IDX]



from __future__ import print_function, division, absolute_import

from test.fixture import SubManFixture
from subscription_manager.gui import networkConfig
import mock
import rhsm.connection as connection
import rhsm.config
import rhsm.utils
import socket
from test import stubs
from nose.plugins.attrib import attr


@attr('gui')
class TestNetworkConfigDialog(SubManFixture):

    def setUp(self):
        SubManFixture.setUp(self)
        self.nc = networkConfig.NetworkConfigDialog()
        self.stubConfig = stubs.StubConfig()

    def test_network_cfg_parse_proxy_entry(self):
        self.nc.enableProxyButton.set_active(True)
        proxy_entry = self.nc.proxyEntry

        proxy_entry.set_text("example.com:10000")
        expected = ('example.com', '10000')
        actual = self.nc.parse_proxy_entry(proxy_entry.get_text())
        self.assertEqual(expected, actual)

        proxy_entry.set_text("http://example.com")
        expected = ('example.com', '3128')
        actual = self.nc.parse_proxy_entry(proxy_entry.get_text())
        self.assertEqual(expected, actual)

        proxy_entry.set_text("http://user@example.com")
        expected = ('example.com', '3128')
        actual = self.nc.parse_proxy_entry(proxy_entry.get_text())
        self.assertEqual(expected, actual)

        proxy_entry.set_text("http://user:pass@example.com")
        expected = ('example.com', '3128')
        actual = self.nc.parse_proxy_entry(proxy_entry.get_text())
        self.assertEqual(expected, actual)

        proxy_entry.set_text("example.com:")
        expected = ('example.com', '3128')
        actual = self.nc.parse_proxy_entry(proxy_entry.get_text())
        self.assertEqual(expected, actual)

    @mock.patch('subscription_manager.gui.networkConfig.rhsm.utils.parse_url')
    def test_network_cfg_parse_proxy_entry_unexpected_exception(self, mock_parse_url):
        self.nc.enableProxyButton.set_active(True)
        proxy_entry = self.nc.proxyEntry
        mock_parse_url.side_effect = rhsm.utils.ServerUrlParseError
        proxy_entry.set_text("example.com:10000")
        expected = ('example.com', '10000')
        actual = self.nc.parse_proxy_entry(proxy_entry.get_text())
        self.assertEqual(expected, actual)

    @mock.patch('subscription_manager.gui.networkConfig.rhsm.utils.parse_url')
    def test_network_cfg_parse_proxy_entry_exceptions(self, mock_parse_url):
        self.nc.enableProxyButton.set_active(True)
        proxy_entry = self.nc.proxyEntry
        mock_parse_url.side_effect = rhsm.utils.ServerUrlParseErrorPort
        proxy_entry.set_text("example.com:")
        expected = ('example.com', rhsm.config.DEFAULT_PROXY_PORT)
        actual = self.nc.parse_proxy_entry(proxy_entry.get_text())
        self.assertEqual(expected, actual)

    def test_network_cfg_set_initial_values(self):
        self.stubConfig = stubs.StubConfig()
        self.nc.set_initial_values()

    def test_network_cfg_write_values(self):
        self.nc = networkConfig.NetworkConfigDialog()
        self.stubConfig = stubs.StubConfig()
        self.nc.cfg = self.stubConfig
        self.nc.enableProxyButton.set_active(True)
        self.nc.proxyEntry.set_text("example.com:10000")
        self.nc.write_values()

    def test_network_cfg_write_values_no_port(self):
        self.nc = networkConfig.NetworkConfigDialog()
        self.stubConfig = stubs.StubConfig()
        self.nc.cfg = self.stubConfig
        self.nc.enableProxyButton.set_active(True)
        self.nc.proxyEntry.set_text("example.com")
        self.nc.write_values()

    def test_network_cfg_write_values_with_auth(self):
        self.nc = networkConfig.NetworkConfigDialog()
        self.stubConfig = stubs.StubConfig()
        self.nc.cfg = self.stubConfig
        self.nc.enableProxyButton.set_active(True)
        self.nc.proxyEntry.set_text("example.com:10000")
        self.nc.enableProxyAuthButton.set_active(True)
        self.nc.proxyUserEntry.set_text("redhatUser")
        self.nc.proxyPasswordEntry.set_text("redhatPass")
        self.nc.write_values()

        actual_user = self.nc.cfg.store['server']['proxy_user']
        actual_password = self.nc.cfg.store['server']['proxy_password']
        self.assertTrue(actual_user == "redhatUser")
        self.assertTrue(actual_password == "redhatPass")

    def test_network_cfg_write_fail(self):
        self.nc = networkConfig.NetworkConfigDialog()
        self.stubConfig = stubs.StubConfig()
        self.stubConfig.raise_io = True
        self.nc.cfg = self.stubConfig
        self.nc.write_values()

    def test_network_cfg_cancel_preserve_values(self):
        self.nc = networkConfig.NetworkConfigDialog()
        self.stubConfig = stubs.StubConfig()
        self.nc.cfg = self.stubConfig
        self.nc.enableProxyButton.set_active(True)
        self.nc.proxyEntry.set_text("example.com:10000")
        self.expected = {}
        self.nc.on_cancel_clicked(self.nc.cancelButton)
        self.assertEqual(self.expected, self.nc.cfg.store)

    def test_network_cfg_save_change_values(self):
        self.nc = networkConfig.NetworkConfigDialog()
        self.stubConfig = stubs.StubConfig()
        self.nc.cfg = self.stubConfig
        self.nc.enableProxyButton.set_active(True)
        self.nc.proxyEntry.set_text("example.com:10000")
        self.expected = {}
        self.nc.on_save_clicked(self.nc.saveButton)
        self.assertNotEqual(self.expected, self.nc.cfg.store)

    @mock.patch('subscription_manager.gui.networkConfig.connection.UEPConnection.getStatus')
    @mock.patch('subscription_manager.gui.networkConfig.socket.setdefaulttimeout')
    def test_network_cfg_preserve_socket_timeout(self,
                                                 mock_setdefaulttimeout,
                                                 mock_connection):
        proxy_host = 'example.com'
        proxy_port = '10000'
        proxy_user = ''
        proxy_password = ''
        no_proxy = ''
        self.nc.test_connection(proxy_host, proxy_port, proxy_user, proxy_password, no_proxy)
        expected = [mock.call(10), mock.call(self.nc.org_timeout)]
        actual = mock_setdefaulttimeout.call_args_list
        self.assertEqual(expected, actual)

    @mock.patch('subscription_manager.gui.networkConfig.connection.UEPConnection.getStatus')
    @mock.patch('subscription_manager.gui.networkConfig.socket.setdefaulttimeout')
    def test_network_cfg_test_connection(self,
                                         mock_setdefaulttimeout,
                                         mock_connection):

        mock_connection.return_value = True
        proxy_host = 'example.com'
        proxy_port = '10000'
        proxy_user = ''
        proxy_password = ''
        no_proxy = ''
        actual = self.nc.test_connection(proxy_host, proxy_port, proxy_user, proxy_password, no_proxy)
        expected = True
        self.assertEqual(expected, actual)

    @mock.patch('subscription_manager.gui.networkConfig.connection.UEPConnection.getStatus')
    @mock.patch('subscription_manager.gui.networkConfig.socket.setdefaulttimeout')
    def test_network_cfg_test_connection_remote_server_exception(self,
                                                                 mock_setdefaulttimeout,
                                                                 mock_connection):

        mock_connection.side_effect = \
        connection.RemoteServerException("RemoteServerException")

        proxy_host = 'example.com'
        proxy_port = '10000'
        proxy_user = ''
        proxy_password = ''
        no_proxy = ''
        actual = self.nc.test_connection(proxy_host, proxy_port, proxy_user, proxy_password, no_proxy)

        expected = True
        self.assertEqual(expected, actual)

    @mock.patch('subscription_manager.gui.networkConfig.connection.UEPConnection.getStatus')
    @mock.patch('subscription_manager.gui.networkConfig.socket.setdefaulttimeout')
    def test_network_cfg_test_connection_restlib_exception(self,
                                                           mock_setdefaulttimeout,
                                                           mock_connection):
        mock_connection.side_effect = \
        connection.RestlibException("RestlibException")

        proxy_host = 'example.com'
        proxy_port = '10000'
        proxy_user = ''
        proxy_password = ''
        no_proxy = ''
        actual = self.nc.test_connection(proxy_host, proxy_port, proxy_user, proxy_password, no_proxy)

        expected = True
        self.assertEqual(expected, actual)

    @mock.patch('subscription_manager.gui.networkConfig.connection.UEPConnection.getStatus')
    @mock.patch('subscription_manager.gui.networkConfig.socket.setdefaulttimeout')
    def test_network_cfg_test_connection_network_exception(self,
                                                           mock_setdefaulttimeout,
                                                           mock_connection):

        mock_connection.side_effect = \
        connection.NetworkException("NetworkException")

        proxy_host = 'example.com'
        proxy_port = '10000'
        proxy_user = ''
        proxy_password = ''
        no_proxy = ''
        actual = self.nc.test_connection(proxy_host, proxy_port, proxy_user, proxy_password, no_proxy)

        expected = False
        self.assertEqual(expected, actual)

    @mock.patch('subscription_manager.gui.networkConfig.connection.UEPConnection.getStatus')
    @mock.patch('subscription_manager.gui.networkConfig.socket.setdefaulttimeout')
    def test_network_cfg_test_connection_unexpected_exception(self,
                                                                mock_setdefaulttimeout,
                                                                mock_connection):

        mock_connection.side_effect = Exception("Made.Up.Exception")

        proxy_host = 'example.com'
        proxy_port = '10000'
        proxy_user = ''
        proxy_password = ''
        no_proxy = ''
        actual = self.nc.test_connection(proxy_host, proxy_port, proxy_user, proxy_password, no_proxy)

        expected = False
        self.assertEqual(expected, actual)

    @mock.patch('subscription_manager.gui.networkConfig.connection.UEPConnection.getStatus')
    @mock.patch('subscription_manager.gui.networkConfig.socket.setdefaulttimeout')
    def test_network_cfg_test_connection_socket_error(self,
                                                      mock_setdefaulttimeout,
                                                      mock_connection):
        mock_connection.side_effect = socket.error("socket.error")

        proxy_host = 'example.com'
        proxy_port = '10000'
        proxy_user = ''
        proxy_password = ''
        no_proxy = ''
        actual = self.nc.test_connection(proxy_host, proxy_port, proxy_user, proxy_password, no_proxy)

        expected = False
        self.assertEqual(expected, actual)

    def test_network_cfg_on_connection_finish(self):
        connection_status_label = self.nc.connectionStatusLabel
        expected = "Proxy connection succeeded"
        self.nc.on_test_connection_finish(True)
        actual = connection_status_label.get_text()
        self.assertEqual(expected, actual)

    def test_network_cfg_on_connection_finish_fail(self):
        connection_status_label = self.nc.connectionStatusLabel
        expected = "Proxy connection failed"
        self.nc.on_test_connection_finish(False)
        actual = connection_status_label.get_text()
        self.assertEqual(expected, actual)

    def test_network_cfg_show(self):
        self.nc.show()

    @mock.patch('subscription_manager.gui.networkConfig.threading.Thread')
    def test_network_cfg_on_test_connection_clicked_auth(self, mock_thread):
        self.nc.enableProxyButton.set_active(True)
        proxy_entry = self.nc.proxyEntry
        proxy_entry.set_text("example.com:10000")
        self.nc.enableProxyAuthButton.set_active(True)
        proxy_user_entry = self.nc.proxyUserEntry
        proxy_user_entry.set_text("redhatUser")
        proxy_password_entry = self.nc.proxyPasswordEntry
        proxy_password_entry.set_text("redhatPass")
        test_connection_button = self.nc.testConnectionButton
        self.nc.on_test_connection_clicked(test_connection_button)
        expected = mock.call(args=('example.com', '10000', 'redhatUser', 'redhatPass', None),
                             name=mock.ANY, target=mock.ANY)
        actual = mock_thread.call_args_list

        self.assertTrue(expected in actual)

    @mock.patch('subscription_manager.gui.networkConfig.threading.Thread')
    def test_network_cfg_on_test_connection_clicked_no_auth(self, mock_thread):
        self.nc.enableProxyButton.set_active(True)
        proxy_entry = self.nc.proxyEntry
        proxy_entry.set_text("example.com:10000")
        self.nc.enableProxyAuthButton.set_active(False)
        proxy_user_entry = self.nc.proxyUserEntry
        proxy_user_entry.set_text("redhatUser")
        proxy_password_entry = self.nc.proxyPasswordEntry
        proxy_password_entry.set_text("redhatPass")
        test_connection_button = self.nc.testConnectionButton
        self.nc.on_test_connection_clicked(test_connection_button)
        expected = mock.call(args=('example.com', '10000', None, None, None), name=mock.ANY, target=mock.ANY)
        actual = mock_thread.call_args_list
        self.assertTrue(expected in actual)



from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2010 Red Hat, Inc.
#
# Authors: Adrian Likins <alikins@redhat.com>
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
import logging

from .certlib import Locker, ActionReport
from subscription_manager import injection as inj

log = logging.getLogger(__name__)


# FactsActionInvoker has a Facts
#   Facts is a CacheManager
class FactsActionInvoker(object):
    """Used by CertActionClient to update a system's facts with the server, used
    primarily by the cron job but in a couple other places as well.

    Makes use of the facts module as well.
    """
    def __init__(self):
        self.locker = Locker()

    def update(self):
        return self.locker.run(self._do_update)

    def _do_update(self):
        action = FactsActionCommand()
        return action.perform()


class FactsActionReport(ActionReport):
    """ActionReport for FactsActionInvoker.

    fact_updates: list of updated facts.
    updates: Number of updated facts.
    """

    name = "Fact updates"

    def __init__(self):
        self.fact_updates = []
        self._exceptions = []
        self._updates = []
        self._status = None

    def updates(self):
        """How many facts were updated."""
        return len(self.fact_updates)


class FactsActionCommand(object):
    """UpdateAction for facts.

    Update facts if calculated local facts are different than
    the cached results of RHSM API known facts.

    If we know facts are now different from out last known
    cache of RHSM API's idea of this consumers facts, update
    the server with the latest version.

    Returns a FactsActionReport.
    """
    def __init__(self):
        cp_provider = inj.require(inj.CP_PROVIDER)
        self.uep = cp_provider.get_consumer_auth_cp()
        self.report = FactsActionReport()
        self.facts = inj.require(inj.FACTS)
        self.facts_client = inj.require(inj.FACTS)

    def perform(self):
        # figure out the diff between latest facts and
        # report that as updates

        if self.facts.has_changed():
            fact_updates = self.facts.get_facts()
            self.report.fact_updates = fact_updates

            consumer_identity = inj.require(inj.IDENTITY)
            if not consumer_identity.is_valid():
                return self.report

            # CacheManager.update_check calls self.has_changed,
            # is the self.facts.has_changed above redundant?
            self.facts.update_check(self.uep, consumer_identity.uuid)
            log.info("Facts have been updated.")
        else:
            log.debug("Facts have not changed, skipping upload.")
        return self.report



from __future__ import print_function, division, absolute_import

# Copyright (c) 2014 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

import subscription_manager.injection as inj

from subscription_manager.model import Content, Entitlement, EntitlementSource


class EntitlementCertContent(Content):
    @classmethod
    def from_cert_content(cls, ent_cert_content, cert=None):
        """
        Creates a generic subscription_manager.model.Content from an
        rhsm.certificate2.Content.

        Because the rhsm Content does not carry a cert (it originates
        from one), we have to pass this in separately.
        """
        return cls(content_type=ent_cert_content.content_type,
            name=ent_cert_content.name, label=ent_cert_content.label,
            url=ent_cert_content.url, gpg=ent_cert_content.gpg,
            tags=ent_cert_content.required_tags, cert=cert,
            enabled=ent_cert_content.enabled,
            metadata_expire=ent_cert_content.metadata_expire)


class EntitlementCertEntitlement(Entitlement):
    """A Entitlement created from an EntitlementCertificate."""
    @classmethod
    def from_ent_cert(cls, ent_cert):
        content_set = []
        for ent_cert_content in ent_cert.content:
            ent_cert_ent_content = EntitlementCertContent.from_cert_content(
                ent_cert_content, ent_cert)
            content_set.append(ent_cert_ent_content)

        # create a :EntitlementCertEntitlement with a EntitledContents
        # as the content Iterables
        ent_cert_ent = cls(content_set, ent_cert.entitlement_type)

        # could populate more info here, but
        # we don't actually seem to use it anywhere
        # here or in repolib
        return ent_cert_ent


class EntitlementDirEntitlementSource(EntitlementSource):
    """Populate with entitlement info from ent dir of ent certs."""

    def __init__(self):
        ent_dir = inj.require(inj.ENT_DIR)
        prod_dir = inj.require(inj.PROD_DIR)

        self.product_tags = prod_dir.get_provided_tags()

        # populate from ent certs
        self._entitlements = []
        for ent_cert in ent_dir.list_valid_with_content_access():
            self._entitlements.append(
                EntitlementCertEntitlement.from_ent_cert(ent_cert))



from __future__ import print_function, division, absolute_import

from nose.plugins.attrib import attr
from mock import patch

try:
    import unittest2 as unittest
except ImportError:
    import unittest

import codecs
import glob
import os

from subscription_manager.i18n import configure_i18n
from subscription_manager.unicode_width import textual_width


class TestI18N(unittest.TestCase):
    def test_configure_i18n(self):
        configure_i18n()

    # Long running test, requires python-kitchen to run
    # determines if python-kitchen replacement gives the
    # same result
    @attr('functional')
    def test_text_width(self):
        from kitchen.text.display import textual_width as kitchen_textual_width
        for po_file in glob.glob('po/*.po'):
            with codecs.open(po_file, 'r', encoding='utf-8') as translation_file:
                for number, line in enumerate(translation_file.readlines(), 1):
                    expected = kitchen_textual_width(line)
                    actual = textual_width(line)
                    self.assertEqual(actual, expected, msg='mismatch on line {} of file {}, {} vs. {}'.format(
                         number, po_file, expected, actual
                    ))

    @patch('subscription_manager.i18n.Locale')
    def test_configure_i18n_lang(self, locale):
        """
        Ensure the method i18n does not pass a None from the environment to Locale
        :return:
        """
        new_lang = 'en-US.UTF-8'

        with patch.dict(os.environ, {'LANG': new_lang}):
            try:
                configure_i18n()
            except:
                self.fail()

        locale.set.assert_called_with(new_lang)

    @patch('subscription_manager.i18n.Locale')
    def test_configure_i18n_lang_none(self, locale):
        """
        Ensure the method i18n does not pass a None from the environment to Locale
        :return:
        """
        with patch.dict(os.environ, clear=True):
            try:
                configure_i18n()
            except:
                self.fail()

        locale.set.assert_not_called()



from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2016 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

import errno

import mock
import json
import dbus.connection
import socket
import six

import subscription_manager.injection as inj

from subscription_manager.cache import InstalledProductsManager
from subscription_manager.cp_provider import CPProvider
from subscription_manager.facts import Facts
from subscription_manager.identity import Identity
from subscription_manager.plugins import PluginManager

from ..fixture import set_up_mock_sp_store

from test.rhsmlib_test.base import DBusObjectTest, InjectionMockingTest

from rhsm import connection

from rhsmlib.dbus import constants
from rhsmlib.dbus.objects import RegisterDBusObject

from rhsmlib.services import register, exceptions

CONSUMER_CONTENT_JSON = '''{"hypervisorId": null,
        "serviceLevel": "",
        "autoheal": true,
        "idCert": {
          "key": "FAKE_KEY",
          "cert": "FAKE_CERT",
          "serial" : {
            "id" : 5196045143213189102,
            "revoked" : false,
            "collected" : false,
            "expiration" : "2033-04-25T18:03:06+0000",
            "serial" : 5196045143213189102,
            "created" : "2017-04-25T18:03:06+0000",
            "updated" : "2017-04-25T18:03:06+0000"
          },
          "id" : "8a8d011e5ba64700015ba647fbd20b88",
          "created" : "2017-04-25T18:03:07+0000",
          "updated" : "2017-04-25T18:03:07+0000"
        },
        "owner": {"href": "/owners/admin", "displayName": "Admin Owner",
        "id": "ff808081550d997c01550d9adaf40003", "key": "admin"},
        "href": "/consumers/c1b8648c-6f0a-4aa5-b34e-b9e62c0e4364",
        "facts": {}, "id": "ff808081550d997c015511b0406d1065",
        "uuid": "c1b8648c-6f0a-4aa5-b34e-b9e62c0e4364",
        "guestIds": null, "capabilities": null,
        "environment": null, "installedProducts": null,
        "canActivate": false, "type": {"manifest": false,
        "id": "1000", "label": "system"}, "annotations": null,
        "username": "admin", "updated": "2016-06-02T15:16:51+0000",
        "lastCheckin": null, "entitlementCount": 0, "releaseVer":
        {"releaseVer": null}, "entitlementStatus": "valid", "name":
        "test.example.com", "created": "2016-06-02T15:16:51+0000",
        "contentTags": null, "dev": false}'''

OWNERS_CONTENT_JSON = '''[
    {
        "autobindDisabled": false,
        "autobindHypervisorDisabled": false,
        "contentAccessMode": "entitlement",
        "contentAccessModeList": "entitlement",
        "contentPrefix": null,
        "created": "2020-02-17T08:21:47+0000",
        "defaultServiceLevel": null,
        "displayName": "Donald Duck",
        "href": "/owners/donaldduck",
        "id": "ff80808170523d030170523d34890003",
        "key": "donaldduck",
        "lastRefreshed": null,
        "logLevel": null,
        "parentOwner": null,
        "updated": "2020-02-17T08:21:47+0000",
        "upstreamConsumer": null
    },
    {
        "autobindDisabled": false,
        "autobindHypervisorDisabled": false,
        "contentAccessMode": "entitlement",
        "contentAccessModeList": "entitlement",
        "contentPrefix": null,
        "created": "2020-02-17T08:21:47+0000",
        "defaultServiceLevel": null,
        "displayName": "Admin Owner",
        "href": "/owners/admin",
        "id": "ff80808170523d030170523d347c0002",
        "key": "admin",
        "lastRefreshed": null,
        "logLevel": null,
        "parentOwner": null,
        "updated": "2020-02-17T08:21:47+0000",
        "upstreamConsumer": null
    },
    {
        "autobindDisabled": false,
        "autobindHypervisorDisabled": false,
        "contentAccessMode": "entitlement",
        "contentAccessModeList": "entitlement,org_environment",
        "contentPrefix": null,
        "created": "2020-02-17T08:21:47+0000",
        "defaultServiceLevel": null,
        "displayName": "Snow White",
        "href": "/owners/snowwhite",
        "id": "ff80808170523d030170523d348a0004",
        "key": "snowwhite",
        "lastRefreshed": null,
        "logLevel": null,
        "parentOwner": null,
        "updated": "2020-02-17T08:21:47+0000",
        "upstreamConsumer": null
    }
]
'''


class RegisterServiceTest(InjectionMockingTest):
    def setUp(self):
        super(RegisterServiceTest, self).setUp()
        self.mock_identity = mock.Mock(spec=Identity, name="Identity")
        self.mock_cp = mock.Mock(spec=connection.UEPConnection, name="UEPConnection")

        # Mock a basic auth connection
        self.mock_cp.username = "username"
        self.mock_cp.password = "password"

        # Add a mock cp_provider
        self.mock_cp_provider = mock.Mock(spec=CPProvider, name="CPProvider")

        # For the tests in which it's used, the consumer_auth cp and basic_auth cp can be the same
        self.mock_cp_provider.get_consumer_auth_cp.return_value = self.mock_cp
        self.mock_cp_provider.get_basic_auth_cp.return_value = self.mock_cp

        self.mock_pm = mock.Mock(spec=PluginManager, name="PluginManager")
        self.mock_installed_products = mock.Mock(spec=InstalledProductsManager,
            name="InstalledProductsManager")
        self.mock_facts = mock.Mock(spec=Facts, name="Facts")
        self.mock_facts.get_facts.return_value = {}

        syspurpose_patch = mock.patch('subscription_manager.syspurposelib.SyncedStore')
        self.mock_sp_store = syspurpose_patch.start()
        self.mock_sp_store, self.mock_sp_store_contents = set_up_mock_sp_store(self.mock_sp_store)
        self.addCleanup(syspurpose_patch.stop)

        self.mock_syspurpose = mock.Mock()

    def injection_definitions(self, *args, **kwargs):
        if args[0] == inj.IDENTITY:
            return self.mock_identity
        elif args[0] == inj.PLUGIN_MANAGER:
            return self.mock_pm
        elif args[0] == inj.INSTALLED_PRODUCTS_MANAGER:
            return self.mock_installed_products
        elif args[0] == inj.FACTS:
            return self.mock_facts
        elif args[0] == inj.CP_PROVIDER:
            return self.mock_cp_provider
        else:
            return None

    @mock.patch("rhsmlib.services.register.syspurposelib.write_syspurpose_cache", return_value=True)
    @mock.patch("rhsmlib.services.register.managerlib.persist_consumer_cert")
    def test_register_normally(self, mock_persist_consumer, mock_write_cache):
        self.mock_identity.is_valid.return_value = False
        self.mock_installed_products.format_for_server.return_value = []
        self.mock_installed_products.tags = []
        expected_consumer = json.loads(CONSUMER_CONTENT_JSON)
        self.mock_cp.registerConsumer.return_value = expected_consumer

        register_service = register.RegisterService(self.mock_cp)
        register_service.register("org", name="name", environment="environment")

        self.mock_cp.registerConsumer.assert_called_once_with(
            name="name",
            facts={},
            owner="org",
            environment="environment",
            keys=None,
            installed_products=[],
            jwt_token=None,
            content_tags=[],
            type="system",
            role="",
            addons=[],
            service_level="",
            usage="")
        self.mock_installed_products.write_cache.assert_called()

        mock_persist_consumer.assert_called_once_with(expected_consumer)
        mock_write_cache.assert_called_once()
        expected_plugin_calls = [
            mock.call('pre_register_consumer', name='name', facts={}),
            mock.call('post_register_consumer', consumer=expected_consumer, facts={})
        ]
        self.assertEqual(expected_plugin_calls, self.mock_pm.run.call_args_list)

    @mock.patch("rhsmlib.services.register.syspurposelib.write_syspurpose_cache", return_value=True)
    @mock.patch("rhsmlib.services.register.managerlib.persist_consumer_cert")
    def test_register_with_activation_keys(self, mock_persist_consumer, mock_write_cache):
        self.mock_cp.username = None
        self.mock_cp.password = None
        self.mock_identity.is_valid.return_value = False
        self.mock_installed_products.format_for_server.return_value = []
        self.mock_installed_products.tags = []

        expected_consumer = json.loads(CONSUMER_CONTENT_JSON)
        self.mock_cp.registerConsumer.return_value = expected_consumer

        register_service = register.RegisterService(self.mock_cp)
        register_service.register("org", name="name", activation_keys=[1])

        self.mock_cp.registerConsumer.assert_called_once_with(
            name="name",
            facts={},
            owner="org",
            environment=None,
            keys=[1],
            installed_products=[],
            jwt_token=None,
            content_tags=[],
            type="system",
            role="",
            addons=[],
            service_level="",
            usage=""
        )
        self.mock_installed_products.write_cache.assert_called()

        mock_persist_consumer.assert_called_once_with(expected_consumer)
        mock_write_cache.assert_called_once()
        expected_plugin_calls = [
            mock.call('pre_register_consumer', name='name', facts={}),
            mock.call('post_register_consumer', consumer=expected_consumer, facts={})
        ]
        self.assertEqual(expected_plugin_calls, self.mock_pm.run.call_args_list)

    @mock.patch("rhsmlib.services.register.syspurposelib.write_syspurpose_cache", return_value=True)
    @mock.patch("rhsmlib.services.register.managerlib.persist_consumer_cert")
    def test_register_with_consumerid(self, mock_persist_consumer, mock_write_cache):
        self.mock_identity.is_valid.return_value = False
        self.mock_installed_products.format_for_server.return_value = []
        self.mock_installed_products.tags = []

        expected_consumer = json.loads(CONSUMER_CONTENT_JSON)
        self.mock_cp.getConsumer.return_value = json.loads(CONSUMER_CONTENT_JSON)

        register_service = register.RegisterService(self.mock_cp)
        register_service.register("org", name="name", consumerid="consumerid")

        self.mock_cp.getConsumer.assert_called_once_with("consumerid")
        self.mock_installed_products.write_cache.assert_called()

        mock_persist_consumer.assert_called_once_with(expected_consumer)
        mock_write_cache.assert_called_once()
        expected_plugin_calls = [
            mock.call('pre_register_consumer', name='name', facts={}),
            mock.call('post_register_consumer', consumer=expected_consumer, facts={})
        ]
        self.assertEqual(expected_plugin_calls, self.mock_pm.run.call_args_list)

    def _build_options(self, activation_keys=None, environment=None, force=None, name=None, consumerid=None):
        return {
            'activation_keys': activation_keys,
            'environment': environment,
            'force': force,
            'name': name,
            'consumerid': consumerid
        }

    def test_fails_when_previously_registered(self):
        self.mock_identity.is_valid.return_value = True

        with self.assertRaisesRegexp(exceptions.ValidationError, r'.*system is already registered.*'):
            register.RegisterService(self.mock_cp).validate_options(self._build_options())

    def test_allows_force(self):
        self.mock_identity.is_valid.return_value = True
        options = self._build_options(force=True)
        register.RegisterService(self.mock_cp).validate_options(options)

    @mock.patch("rhsmlib.services.register.syspurposelib.write_syspurpose_cache", return_value=True)
    @mock.patch("rhsmlib.services.register.managerlib.persist_consumer_cert")
    def test_reads_syspurpose(self, mock_persist_consumer, mock_write_cache):
        self.mock_installed_products.format_for_server.return_value = []
        self.mock_installed_products.tags = []
        self.mock_identity.is_valid.return_value = False
        self.mock_sp_store_contents["role"] = "test_role"
        self.mock_sp_store_contents["service_level_agreement"] = "test_sla"
        self.mock_sp_store_contents["addons"] = ["addon1"]
        self.mock_sp_store_contents["usage"] = "test_usage"

        expected_consumer = json.loads(CONSUMER_CONTENT_JSON)
        self.mock_cp.registerConsumer.return_value = expected_consumer

        register_service = register.RegisterService(self.mock_cp)
        register_service.register("org", name="name")

        self.mock_cp.registerConsumer.assert_called_once_with(
            addons=["addon1"],
            content_tags=[],
            environment=None,
            facts={},
            installed_products=[],
            jwt_token=None,
            keys=None,
            name="name",
            owner="org",
            role="test_role",
            service_level="test_sla",
            type="system",
            usage="test_usage")
        mock_write_cache.assert_called_once()

    def test_does_not_require_basic_auth_with_activation_keys(self):
        self.mock_cp.username = None
        self.mock_cp.password = None

        self.mock_identity.is_valid.return_value = False
        options = self._build_options(activation_keys=[1])
        register.RegisterService(self.mock_cp).validate_options(options)

    def test_does_not_allow_basic_auth_with_activation_keys(self):
        self.mock_identity.is_valid.return_value = False
        options = self._build_options(activation_keys=[1])
        with self.assertRaisesRegexp(exceptions.ValidationError, r'.*do not require user credentials.*'):
            register.RegisterService(self.mock_cp).validate_options(options)

    def test_does_not_allow_environment_with_activation_keys(self):
        self.mock_cp.username = None
        self.mock_cp.password = None

        self.mock_identity.is_valid.return_value = False
        options = self._build_options(activation_keys=[1], environment='environment')
        with self.assertRaisesRegexp(exceptions.ValidationError, r'.*do not allow environments.*'):
            register.RegisterService(self.mock_cp).validate_options(options)

    def test_does_not_allow_environment_with_consumerid(self):
        self.mock_cp.username = None
        self.mock_cp.password = None

        self.mock_identity.is_valid.return_value = False
        options = self._build_options(activation_keys=[1], consumerid='consumerid')
        with self.assertRaisesRegexp(exceptions.ValidationError, r'.*previously registered.*'):
            register.RegisterService(self.mock_cp).validate_options(options)

    def test_requires_basic_auth_for_normal_registration(self):
        self.mock_cp.username = None
        self.mock_cp.password = None

        self.mock_identity.is_valid.return_value = False
        options = self._build_options(consumerid='consumerid')
        with self.assertRaisesRegexp(exceptions.ValidationError, r'.*Missing username.*'):
            register.RegisterService(self.mock_cp).validate_options(options)


class DomainSocketRegisterDBusObjectTest(DBusObjectTest, InjectionMockingTest):
    def dbus_objects(self):
        return [RegisterDBusObject]

    def setUp(self):
        super(DomainSocketRegisterDBusObjectTest, self).setUp()

        self.proxy = self.proxy_for(RegisterDBusObject.default_dbus_path)
        self.interface = dbus.Interface(self.proxy, constants.REGISTER_INTERFACE)

        self.mock_identity.is_valid.return_value = True

        self.mock_cp_provider = mock.Mock(spec=CPProvider, name="CPProvider")
        self.mock_cp = mock.Mock(spec=connection.UEPConnection, name="UEPConnection")

        # Mock a basic auth connection
        self.mock_cp.username = "username"
        self.mock_cp.password = "password"

        self.mock_cp.getOwnerList = mock.Mock()
        self.mock_cp.getOwnerList.return_value = json.loads(OWNERS_CONTENT_JSON)

        # For the tests in which it's used, the consumer_auth cp and basic_auth cp can be the same
        self.mock_cp_provider.get_consumer_auth_cp.return_value = self.mock_cp
        self.mock_cp_provider.get_basic_auth_cp.return_value = self.mock_cp

        register_patcher = mock.patch('rhsmlib.dbus.objects.register.RegisterService', autospec=True)
        self.mock_register = register_patcher.start().return_value
        self.addCleanup(register_patcher.stop)

        cert_invoker_patcher = mock.patch('rhsmlib.dbus.objects.register.EntCertActionInvoker', autospec=True)
        self.mock_cert_invoker = cert_invoker_patcher.start().return_value
        self.addCleanup(cert_invoker_patcher.stop)

    def injection_definitions(self, *args, **kwargs):
        if args[0] == inj.IDENTITY:
            return self.mock_identity
        elif args[0] == inj.CP_PROVIDER:
            return self.mock_cp_provider
        else:
            return None

    def test_open_domain_socket(self):
        dbus_method_args = ['']

        def assertions(*args):
            result = args[0]
            six.assertRegex(self, result, r'/run/dbus.*')

        self.dbus_request(assertions, self.interface.Start, dbus_method_args)

    def test_same_socket_on_subsequent_opens(self):
        dbus_method_args = ['']

        def assertions(*args):
            # Assign the result as an attribute to this function.
            # See http://stackoverflow.com/a/27910553/6124862
            assertions.result = args[0]
            six.assertRegex(self, assertions.result, r'/run/dbus.*')

        self.dbus_request(assertions, self.interface.Start, dbus_method_args)

        # Reset the handler_complete_event so we'll block for the second
        # dbus_request
        self.handler_complete_event.clear()

        def assertions2(*args):
            result2 = args[0]
            self.assertEqual(assertions.result, result2)

        self.dbus_request(assertions2, self.interface.Start, dbus_method_args)

    def test_cannot_close_what_is_not_opened(self):
        dbus_method_args = ['']
        with self.assertRaises(dbus.exceptions.DBusException):
            self.dbus_request(None, self.interface.Stop, dbus_method_args)

    def test_closes_domain_socket(self):
        dbus_method_args = ['']

        def get_address(*args):
            address = args[0]
            _prefix, _equal, address = address.partition('=')
            get_address.address, _equal, _suffix = address.partition(',')

        self.dbus_request(get_address, self.interface.Start, dbus_method_args)
        self.handler_complete_event.clear()

        sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        try:
            # The socket returned for connection is an abstract socket so we have
            # to begin the name with a NUL byte to get into that namespace.  See
            # http://blog.eduardofleury.com/archives/2007/09/13
            sock.connect('\0' + get_address.address)
        finally:
            sock.close()

        self.dbus_request(None, self.interface.Stop, dbus_method_args)
        self.handler_complete_event.wait()

        with self.assertRaises(socket.error) as serr:
            sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
            try:
                sock.connect('\0' + get_address.address)
            finally:
                sock.close()
            self.assertEqual(serr.errno, errno.ECONNREFUSED)

    def _build_interface(self):
        dbus_method_args = ['']

        def get_address(*args):
            get_address.address = args[0]

        self.dbus_request(get_address, self.interface.Start, dbus_method_args)
        self.handler_complete_event.clear()
        socket_conn = dbus.connection.Connection(get_address.address)
        socket_proxy = socket_conn.get_object(constants.BUS_NAME, constants.PRIVATE_REGISTER_DBUS_PATH)
        return dbus.Interface(socket_proxy, constants.PRIVATE_REGISTER_INTERFACE)

    def test_can_register_over_domain_socket(self):
        expected_consumer = json.loads(CONSUMER_CONTENT_JSON)

        def assertions(*args):
            # Be sure we are persisting the consumer cert
            self.assertEqual(json.loads(args[0]), expected_consumer)

        self.mock_identity.is_valid.return_value = False
        self.mock_identity.uuid = 'INVALIDCONSUMERUUID'

        self.mock_register.register.return_value = expected_consumer

        dbus_method_args = ['admin', 'admin', 'admin', {}, {}, '']
        self.dbus_request(assertions, self._build_interface().Register, dbus_method_args)

    def test_can_get_orgs_over_domain_socket(self):
        expected_owners = json.loads(OWNERS_CONTENT_JSON)
        expected_consumer = json.loads(CONSUMER_CONTENT_JSON)

        def assertions(*args):
            # Be sure we are persisting the consumer cert
            self.assertEqual(json.loads(args[0]), expected_owners)

        self.mock_identity.is_valid.return_value = False
        self.mock_identity.uuid = 'INVALIDCONSUMERUUID'

        self.mock_register.register.return_value = expected_consumer

        dbus_method_args = ['admin', 'admin', {}, '']
        self.dbus_request(assertions, self._build_interface().GetOrgs, dbus_method_args)

    def test_can_register_over_domain_socket_with_activation_keys(self):
        expected_consumer = json.loads(CONSUMER_CONTENT_JSON)

        def assertions(*args):
            # Be sure we are persisting the consumer cert
            self.assertEqual(json.loads(args[0]), expected_consumer)

        self.mock_identity.is_valid.return_value = False
        self.mock_identity.uuid = 'INVALIDCONSUMERUUID'

        self.mock_register.register.return_value = expected_consumer

        dbus_method_args = [
            'admin',
            ['key1', 'key2'],
            {},
            {
                'host': 'localhost',
                'port': '8443',
                'handler': '/candlepin'
            },
            ''
        ]

        self.dbus_request(assertions, self._build_interface().RegisterWithActivationKeys, dbus_method_args)



# -*- coding: utf-8 -*-

from __future__ import print_function, division, absolute_import
#
# Copyright (c) 2018 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.

import argparse
import os
import sys
import six

from syspurpose.files import SyncedStore
from syspurpose.utils import in_container, make_utf8
from syspurpose.i18n import ugettext as _
import json


SP_CONFLICT_MESSAGE = _("Warning: A {attr} of \"{download_value}\" was recently set for this system "
                        "by the entitlement server administrator.\n{advice}")
SP_ADVICE = _("If you'd like to overwrite the server side change please run: {command}")


def add_command(args, syspurposestore):
    """
    Uses the syspurposestore to add one or more values to a particular property.
    :param args: The parsed args from argparse, expected attributes are:
        prop_name: the string name of the property to add to
        values: A list of the values to add to the given property (could be anything json-serializable)
    :param syspurposestore: An SyspurposeStore object to manipulate
    :return: None
    """
    any_prop_added = False
    for value in args.values:
        if syspurposestore.add(args.prop_name, value):
            any_prop_added = True
            print(_("Added {value} to {prop_name}.").format(
                value=make_utf8(value), prop_name=make_utf8(args.prop_name)))
        else:
            print(_("Not adding value {value} to {prop_name}; it already exists.").format(
                value=make_utf8(value), prop_name=make_utf8(args.prop_name)))

    if any_prop_added is False:
        return

    success_msg = _("{attr} updated.").format(attr=make_utf8(args.prop_name))
    to_add = "".join(args.values)
    command = "syspurpose add-{name} ".format(name=args.prop_name) + to_add
    check_result(
        syspurposestore,
        expectation=lambda res: all(x in res.get(args.prop_name, []) for x in args.values),
        success_msg=success_msg,
        command=command,
        attr=args.prop_name
    )


def remove_command(args, syspurposestore):
    """
    Uses the syspurposestore to remove one or more values from a particular property.
    :param args: The parsed args from argparse, expected attributes are:
        prop_name: the string name of the property to add to
        values: A list of the values to remove from the given property (could be anything json-serializable)
    :param syspurposestore: An SyspurposeStore object to manipulate
    :return: None
    """
    for value in args.values:
        if syspurposestore.remove(args.prop_name, value):
            print(_("Removed \"{val}\" from {name}.").format(val=make_utf8(value), name=make_utf8(args.prop_name)))
        else:
            print(_("Not removing value \"{val}\" from {name}; it was not there.").format(
                val=make_utf8(value), name=make_utf8(args.prop_name)))
            return

    success_msg = _("{attr} updated.").format(attr=make_utf8(args.prop_name))
    to_remove = "".join(args.values)
    command = "syspurpose remove-{name} ".format(name=args.prop_name) + to_remove
    check_result(
        syspurposestore,
        expectation=lambda res: all(x not in res.get('addons', []) for x in args.values),
        success_msg=success_msg,
        command=command,
        attr=args.prop_name
    )


def set_command(args, syspurposestore):
    """
    Uses the syspurposestore to set the prop_name to value.
    :param args: The parsed args from argparse, expected attributes are:
        prop_name: the string name of the property to set
        value: An object to set the property to (could be anything json-serializable)
    :param syspurposestore: An SyspurposeStore object to manipulate
    :return: None
    """
    syspurposestore.set(args.prop_name, args.value)

    success_msg = _("{attr} set to \"{val}\".").format(attr=make_utf8(args.prop_name), val=make_utf8(args.value))
    check_result(
        syspurposestore,
        expectation=lambda res: res.get(args.prop_name) == args.value,
        success_msg=success_msg,
        command="syspurpose set {name} \"{val}\"".format(
            name=args.prop_name, val=args.value),
        attr=args.prop_name
    )


def unset_command(args, syspurposestore):
    """
    Uses the syspurposestore to unset (clear entirely) the prop_name.
    :param args: The parsed args from argparse, expected attributes are:
        prop_name: the string name of the property to unset (clear)
    :param syspurposestore: An SyspurposeStore object to manipulate
    :return: None
    """
    syspurposestore.unset(args.prop_name)

    success_msg = _("{attr} unset.").format(attr=make_utf8(args.prop_name))
    check_result(
        syspurposestore,
        expectation=lambda res: res.get(args.prop_name) in ["", None, []],
        success_msg=success_msg,
        command="syspurpose unset {name}".format(name=args.prop_name),
        attr=args.prop_name
    )


def show_contents(args, syspurposestore):
    """
    :param args:
    :param syspurposestore:
    :return:
    """
    sync_result = syspurposestore.sync()
    contents = sync_result.result
    contents = {key: contents[key] for key in contents if contents[key]}
    print(json.dumps(contents, indent=2, ensure_ascii=False, sort_keys=True))
    return sync_result


def setup_arg_parser():
    """
    Sets up argument parsing for the syspurpose tool.
    :return: An argparse.ArgumentParser ready to use to parse_args
    """
    parser = argparse.ArgumentParser(prog="syspurpose", description="System Syspurpose Management Tool")

    subparsers = parser.add_subparsers(help="sub-command help")

    # Arguments shared by subcommands
    add_options = argparse.ArgumentParser(add_help=False)
    add_options.add_argument("values", help="The value(s) to add", nargs='+')
    add_options.set_defaults(func=add_command, requires_sync=True)

    remove_options = argparse.ArgumentParser(add_help=False)
    remove_options.add_argument("values", help="The value(s) to remove", nargs='+')
    remove_options.set_defaults(func=remove_command, requires_sync=True)

    set_options = argparse.ArgumentParser(add_help=False)
    set_options.add_argument("value", help="The value to set", action="store")
    set_options.set_defaults(func=set_command, requires_sync=True)

    unset_options = argparse.ArgumentParser(add_help=False)
    unset_options.set_defaults(func=unset_command, requires_sync=True)

    # Generic assignments
    # Set ################
    generic_set_parser = subparsers.add_parser(
        "set",
        help=_("Sets the value for the given property")
    )

    generic_set_parser.add_argument(
        "prop_name",
        metavar="property",
        help=_("The name of the property to set/update"),
        action="store"
    )

    generic_set_parser.add_argument(
        "value",
        help=_("The value to set"),
        action="store"
    )

    generic_set_parser.set_defaults(func=set_command, requires_sync=True)

    # Unset ##############
    generic_unset_parser = subparsers.add_parser(
        "unset",
        help=_("Unsets (clears) the value for the given property"),
        parents=[unset_options]
    )

    generic_unset_parser.add_argument(
        "prop_name",
        metavar="property",
        help=_("The name of the property to set/update"),
        action="store"
    )

    # Add ################
    generic_add_parser = subparsers.add_parser(
        "add",
        help=_("Adds the value(s) to the given property")
    )

    generic_add_parser.add_argument(
        "prop_name",
        metavar="property",
        help=_("The name of the property to update"),
        action="store"
    )

    generic_add_parser.add_argument(
        "values",
        help=_("The value(s) to add"),
        action="store",
        nargs="+"
    )

    generic_add_parser.set_defaults(func=add_command, requires_sync=True)

    # Remove #############
    generic_remove_parser = subparsers.add_parser(
        "remove",
        help=_("Removes the value(s) from the given property")
    )

    generic_remove_parser.add_argument(
        "prop_name",
        metavar="property",
        help=_("The name of the property to update"),
        action="store"
    )

    generic_remove_parser.add_argument(
        "values",
        help=_("The value(s) to remove"),
        action="store",
        nargs="+"
    )

    generic_remove_parser.set_defaults(func=remove_command, requires_sync=True)
    # Targeted commands
    # Roles ##########
    set_role_parser = subparsers.add_parser(
        "set-role",
        help=_("Set the system role to the system syspurpose"),
        parents=[set_options]
    )
    # TODO: Set prop_name from schema file
    set_role_parser.set_defaults(prop_name="role")

    unset_role_parser = subparsers.add_parser(
        "unset-role",
        help=_("Clear set role"),
        parents=[unset_options]
    )
    unset_role_parser.set_defaults(prop_name="role")

    # ADDONS #############
    add_addons_parser = subparsers.add_parser(
        "add-addons",
        help=_("Add addons to the system syspurpose"),
        parents=[add_options]
    )
    # TODO: Set prop_name from schema file
    add_addons_parser.set_defaults(prop_name="addons")

    remove_addons_parser = subparsers.add_parser(
        "remove-addons",
        help=_("Remove addons from the system syspurpose"),
        parents=[remove_options]
    )
    remove_addons_parser.set_defaults(prop_name="addons")

    unset_role_parser = subparsers.add_parser(
        "unset-addons",
        help=_("Clear set addons"),
        parents=[unset_options]
    )
    unset_role_parser.set_defaults(prop_name="addons")

    # SLA ################
    set_sla_parser = subparsers.add_parser(
        "set-sla",
        help=_("Set the system sla"),
        parents=[set_options]
    )
    set_sla_parser.set_defaults(prop_name="service_level_agreement")

    unset_sla_parser = subparsers.add_parser(
        "unset-sla",
        help=_("Clear set sla"),
        parents=[unset_options]
    )
    unset_sla_parser.set_defaults(prop_name="service_level_agreement")

    # USAGE ##############
    set_usage_parser = subparsers.add_parser(
        "set-usage",
        help=_("Set the system usage"),
        parents=[set_options]
    )

    set_usage_parser.set_defaults(prop_name="usage")

    unset_usage_parser = subparsers.add_parser(
        "unset-usage",
        help=_("Clear set usage"),
        parents=[unset_options]
    )
    unset_usage_parser.set_defaults(prop_name="usage")

    # Pretty Print Json contents of default syspurpose file
    show_parser = subparsers.add_parser(
        "show",
        help=_("Show the current system syspurpose")
    )
    show_parser.set_defaults(func=show_contents, requires_sync=False)

    return parser


def main():
    """
    Run the cli (Do the syspurpose tool thing!!)
    :return: 0
    """

    parser = setup_arg_parser()
    args = parser.parse_args()

    # Syspurpose is not intended to be used in containers for the time being (could change later).
    if in_container():
        print(_("WARNING: Setting syspurpose in containers has no effect."
              "Please run syspurpose on the host.\n"))

    try:
        from subscription_manager.identity import Identity
        from subscription_manager.cp_provider import CPProvider
        identity = Identity()
        uuid = identity.uuid
        uep = CPProvider().get_consumer_auth_cp()
    except ImportError:
        uuid = None
        uep = None
        print(_("Warning: Unable to sync system purpose with subscription management server:"
                " subscription_manager module is not available."))

    syspurposestore = SyncedStore(uep=uep, consumer_uuid=uuid, use_valid_fields=True)
    if getattr(args, 'func', None) is not None:
        args.func(args, syspurposestore)
    else:
        parser.print_help()
        return 0

    return 0


def system_exit(code, msgs=None):
    """
    Exit with a code and optional message(s). Saved a few lines of code.
    Note: this method was copied from subscription_manager, because syspurpose should be independent
    on subscription_manager module as much as possible.
    """

    if msgs:
        if type(msgs) not in [type([]), type(())]:
            msgs = (msgs, )
        for msg in msgs:
            if isinstance(msg, Exception):
                msg = "%s" % msg

            if isinstance(msg, six.text_type) and six.PY2:
                sys.stderr.write(msg.encode("utf8"))
                sys.stderr.write("\n")
            else:
                sys.stderr.write(msg)
                sys.stderr.write("\n")

    try:
        sys.stdout.flush()
        sys.stderr.flush()
    except IOError:
        pass

    sys.exit(code)


def check_result(syspurposestore, expectation, success_msg, command, attr):
    if syspurposestore:
        syspurposestore.sync()
        result = syspurposestore.get_cached_contents()
    else:
        result = {}
    if result and not expectation(result):
        advice = SP_ADVICE.format(command=command)
        value = result[attr]
        system_exit(
            os.EX_SOFTWARE,
            msgs=_(SP_CONFLICT_MESSAGE.format(
                attr=attr,
                download_value=value,
                advice=advice
            ))
        )
    else:
        print(_(success_msg))



from __future__ import print_function, division, absolute_import

# Copyright (c) 2012 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.

import unittest

from rhsm.huffman import HuffmanNode


class TestHuffmanNode(unittest.TestCase):
    def setUp(self):
        self.node1 = HuffmanNode(1)
        self.node2 = HuffmanNode(2)
        self.parent = HuffmanNode.combine(self.node1, self.node2)

    def test_combine(self):
        self.assertEqual(self.parent.weight, 3)
        self.assertEqual(self.parent.left, self.node1)
        self.assertEqual(self.parent.right, self.node2)
        self.assertEqual(self.node1.parent, self.parent)
        self.assertEqual(self.node2.parent, self.parent)

    def test_build_tree(self):
        leaves = [HuffmanNode(weight) for weight in range(1, 5)]
        root = HuffmanNode.build_tree(leaves)

        # assertions based on calculating these by hand
        self.assertEqual(root.weight, 10)
        self.assertEqual(leaves[0].code, '110')
        self.assertEqual(leaves[1].code, '111')
        self.assertEqual(leaves[2].code, '10')
        self.assertEqual(leaves[3].code, '0')

        for leaf in leaves:
            self.assertTrue(leaf.is_leaf)

    def test_code_non_leaf(self):
        self.assertRaises(AttributeError, getattr, self.parent, 'code')

    def test_is_leaf(self):
        self.assertTrue(self.node1.is_leaf)
        self.assertTrue(self.node2.is_leaf)
        self.assertFalse(self.parent.is_leaf)

    def test_direction_from_parent_left(self):
        self.assertEqual(self.node1.direction_from_parent, '0')

    def test_direction_from_parent_right(self):
        self.node1.right = self.node2
        self.assertEqual(self.node2.direction_from_parent, '1')

    def test_compare_lt(self):
        self.assertTrue(self.node1 < self.node2)

    def test_compare_gt(self):
        self.assertTrue(self.node2 > self.node1)

    def test_compare_eq(self):
        node = HuffmanNode(3)
        self.assertEqual(self.parent, node)

    def test_root_weight(self):
        # sanity check that total weight should be sum of all node weights
        for n in range(4, 100):
            leaves = [HuffmanNode(weight) for weight in range(1, n)]
            tree = HuffmanNode.build_tree(leaves)
            self.assertEqual(tree.weight, sum(leaf.weight for leaf in leaves))



# -*- coding: utf-8 -*-

from __future__ import print_function, division, absolute_import
#
# Copyright (c) 2018 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.

from mock import MagicMock, patch

# Mock importing subscription_manager modules, because we do not want to print
# anything to log or exit the testing application in tests
import sys
sys.modules['subscription_manager'] = MagicMock()
sys.modules['subscription_manager.cli'] = MagicMock()

from syspurpose import cli, files
from .base import SyspurposeTestBase, Capture
import os


class SyspurposeCliTests(SyspurposeTestBase):

    def setUp(self):
        self.OLD_PATH = files.SyncedStore.PATH
        self.OLD_CACHE_PATH = files.SyncedStore.CACHE_PATH
        self.tmp_dir = self._mktmp()
        self.tmp_cache_dir = self._mktmp()

        user_syspurpose_dir_patch = patch('syspurpose.files.USER_SYSPURPOSE_DIR', self.tmp_dir)
        user_syspurpose_dir_patch.start()
        self.addCleanup(user_syspurpose_dir_patch.stop)

        cache_dir_patch = patch('syspurpose.files.CACHE_DIR', self.tmp_cache_dir)
        cache_dir_patch.start()
        self.addCleanup(cache_dir_patch.stop)

        files.SyncedStore.PATH = os.path.join(self.tmp_dir, 'syspurpose.json')
        with open(files.SyncedStore.PATH, "w") as fp:
            fp.write("{}")
        files.SyncedStore.CACHE_PATH = os.path.join(self.tmp_cache_dir, 'syspurpose.json')
        with open(files.SyncedStore.CACHE_PATH, "w") as fp:
            fp.write("{}")
        self.syspurposestore = files.SyncedStore(uep=None, consumer_uuid=None)

    def tearDown(self):
        files.SyncedStore.PATH = self.OLD_PATH
        files.SyncedStore.CACHE_PATH = self.OLD_CACHE_PATH

    def test_unset_command(self):
        """
        A smoke test to ensure nothing bizarre happens when we try to unset some attribute
        """
        args = MagicMock()
        args.prop_name = "foo"
        with Capture() as captured:
            cli.unset_command(args, self.syspurposestore)
            self.assertTrue('foo unset' in captured.out)

    def test_set_command(self):
        """
        A smoke test to ensure nothing bizarre happens when we try to set some attribute
        """
        args = MagicMock()
        args.prop_name = "foo"
        args.value = "bar"
        with Capture() as captured:
            cli.set_command(args, self.syspurposestore)
            self.assertTrue('foo set to "bar"' in captured.out)

    def test_add_command_one_value(self):
        """
        A smoke test to ensure nothing bizarre happens when we try to add some attribute
        """
        args = MagicMock()
        args.prop_name = "addons"
        args.values = ["ADDON1"]
        with Capture() as captured:
            cli.add_command(args, self.syspurposestore)
            self.assertTrue('Added ADDON1 to addons' in captured.out)

    def test_add_generic_command_one_value(self):
        """
        A smoke test to ensure nothing bizarre happens when we try to add some attribute (not addons)
        """
        args = MagicMock()
        args.prop_name = "foo"
        args.values = ["BAR1"]
        with Capture() as captured:
            cli.add_command(args, self.syspurposestore)
            self.assertTrue('Added BAR1 to foo' in captured.out)

    def test_add_command_more_values(self):
        """
        A smoke test to ensure nothing bizarre happens when we try to add more addons attribute
        """
        args = MagicMock()
        args.prop_name = "addons"
        args.values = ["ADDON1", "ADDON2"]
        with Capture() as captured:
            cli.add_command(args, self.syspurposestore)
            self.assertTrue('Added ADDON1 to addons' in captured.out)
            self.assertTrue('Added ADDON2 to addons' in captured.out)
            self.assertTrue('addons updated.' in captured.out)

    def test_add_generic_command_more_value(self):
        """
        A smoke test to ensure nothing bizarre happens when we try to add more generic list attribute
        """
        args = MagicMock()
        args.prop_name = "foo"
        args.values = ["BAR1", "BAR2"]
        with Capture() as captured:
            cli.add_command(args, self.syspurposestore)
            self.assertTrue('Added BAR1 to foo' in captured.out)
            self.assertTrue('Added BAR2 to foo' in captured.out)
            self.assertTrue('foo updated.' in captured.out)

    def test_add_command_existing_values(self):
        """
        A smoke test to ensure nothing bizarre happens when we try to add existing addons attribute
        """
        args = MagicMock()
        args.prop_name = "addons"
        args.values = ["ADDON1", "ADDON2"]
        with Capture() as captured:
            cli.add_command(args, self.syspurposestore)
            self.assertTrue('Added ADDON1 to addons' in captured.out)
            self.assertTrue('Added ADDON2 to addons' in captured.out)
            self.assertTrue('addons updated.' in captured.out)
        # Try to add same addons once again
        args.prop_name = "addons"
        args.values = ["ADDON1", "ADDON2"]
        with Capture() as captured:
            cli.add_command(args, self.syspurposestore)
            self.assertTrue('Not adding value ADDON1 to addons; it already exists.' in captured.out)
            self.assertTrue('Not adding value ADDON2 to addons; it already exists.' in captured.out)
            self.assertFalse('addons updated.' in captured.out)

    def test_add_command_existing_values_and_one_new(self):
        """
        A smoke test to ensure nothing bizarre happens when we try to add existing addons attribute
        and also try to add one new value
        """
        args = MagicMock()
        args.prop_name = "addons"
        args.values = ["ADDON1", "ADDON2"]
        with Capture() as captured:
            cli.add_command(args, self.syspurposestore)
            self.assertTrue('Added ADDON1 to addons' in captured.out)
            self.assertTrue('Added ADDON2 to addons' in captured.out)
            self.assertTrue('addons updated.' in captured.out)
        # Try to add same addons once again
        args.prop_name = "addons"
        args.values = ["ADDON1", "ADDON2", "ADDON3"]
        with Capture() as captured:
            cli.add_command(args, self.syspurposestore)
            self.assertTrue('Not adding value ADDON1 to addons; it already exists.' in captured.out)
            self.assertTrue('Not adding value ADDON2 to addons; it already exists.' in captured.out)
            self.assertTrue('Added ADDON3 to addons' in captured.out)
            self.assertTrue('addons updated.' in captured.out)

    def test_remove_command(self):
        """
        A smoke test to ensure nothing bizarre happens when we try to remove one value from addons attribute
        """
        args = MagicMock()

        # Add value first
        args.prop_name = "addons"
        args.values = ["ADDON1"]
        cli.add_command(args, self.syspurposestore)

        # Now we can try to remove value
        with Capture() as captured:
            cli.remove_command(args, self.syspurposestore)
            self.assertTrue('Removed "ADDON1" from addons' in captured.out)

    def test_generic_remove_command(self):
        """
        A smoke test to ensure nothing bizarre happens when we try to remove one value from generic list
        """
        args = MagicMock()

        # Add value first
        args.prop_name = "foo"
        args.values = ["BAR1"]
        cli.add_command(args, self.syspurposestore)

        # Now we can try to remove value
        with Capture() as captured:
            cli.remove_command(args, self.syspurposestore)
            self.assertTrue('Removed "BAR1" from foo' in captured.out)

    def test_show_command(self):
        """
        A smoke test to ensure nothing bizarre happens when we try to show content fo syspurpose.json file
        """
        args = MagicMock()

        # Add value first
        with Capture() as captured:
            cli.show_contents(args, self.syspurposestore)
            self.assertTrue('{}' in captured.out)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2020 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

# TODO: test Python3 syntax using flake8
# flake8: noqa

"""
This module implements the base class for detecting cloud provider
"""


class CloudDetector(object):
    """
    Base class used for detecting cloud provider
    """

    ID = None

    def __init__(self, hw_info):
        """
        Initialize cloud detector
        """
        self.hw_info = hw_info

    def is_vm(self) -> bool:
        """
        Is current system virtual machine?
        :return: Return True, when it is virtual machine; otherwise return False
        """
        return 'virt.is_guest' in self.hw_info and self.hw_info['virt.is_guest'] is True

    def is_running_on_cloud(self) -> bool:
        """
        Try to guess cloud provider using collected hardware information (output of dmidecode, virt-what, etc.)
        :return: True, when we detected sign of cloud provider in hw info; Otherwise return False
        """
        raise NotImplementedError

    def is_likely_running_on_cloud(self) -> float:
        """
        When all subclasses cannot detect cloud provider using method is_running_on_cloud, because cloud provider
        started to provide something else in output of dmidecode, then try to use this heuristics method
        :return: Float value representing probability that vm is running using specific cloud provider
        """
        raise NotImplementedError



from __future__ import print_function, division, absolute_import

# Copyright (c) 2016 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
try:
    import unittest2 as unittest
except ImportError:
    import unittest

import dbus
import mock

from rhsmlib.dbus import service_wrapper, constants


class ServiceWrapperTest(unittest.TestCase):
    def test_parse_argv(self):
        opts, args = service_wrapper.parse_argv(
            ['cmd_name', '--verbose', '--bus-name', 'Hello', 'Foo'], 'Default')
        self.assertTrue(opts.verbose)
        self.assertEqual(opts.bus_name, 'Hello')
        self.assertEqual(args, ['Foo'])

    def test_uses_default_bus_name(self):
        opts, args = service_wrapper.parse_argv(['cmd_name', 'Foo'], 'Default')
        self.assertFalse(opts.verbose)
        self.assertEqual(opts.bus, dbus.SystemBus)
        self.assertEqual(opts.bus_name, 'Default')
        self.assertEqual(args, ['Foo'])

    def test_loads_bus_given(self):
        opts, args = service_wrapper.parse_argv(['cmd_name', '--bus', 'dbus.SessionBus', 'Foo'], 'Default')
        self.assertEqual(opts.bus, dbus.SessionBus)

    @mock.patch("rhsmlib.dbus.service_wrapper.server.Server")
    def test_loads_an_object_class(self, mock_serve):
        # Just use some class we have available
        service_wrapper.main(['cmd_name', 'mock.MagicMock'])
        mock_serve.assert_called_with(
            bus_class=dbus.SystemBus,
            bus_name=constants.BUS_NAME,
            object_classes=[mock.MagicMock]
        )

    @mock.patch("rhsmlib.dbus.service_wrapper.server.Server")
    def test_loads_from_an_array_of_classes(self, mock_serve):
        service_wrapper.main(['cmd_name'], [mock.MagicMock])
        mock_serve.assert_called_with(
            bus_class=dbus.SystemBus,
            bus_name=constants.BUS_NAME,
            object_classes=[mock.MagicMock]
        )



from __future__ import print_function, division, absolute_import

# Copyright (c) 2017 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
import logging
import socket

from rhsmlib.services import exceptions

from subscription_manager import injection as inj
from subscription_manager import managerlib
from subscription_manager import syspurposelib
from subscription_manager.i18n import ugettext as _

log = logging.getLogger(__name__)


class RegisterService(object):
    def __init__(self, cp):
        self.plugin_manager = inj.require(inj.PLUGIN_MANAGER)
        self.installed_mgr = inj.require(inj.INSTALLED_PRODUCTS_MANAGER)
        self.facts = inj.require(inj.FACTS)
        self.identity = inj.require(inj.IDENTITY)
        self.cp = cp

    def register(self, org, activation_keys=None, environment=None, force=None, name=None, consumerid=None,
            type=None, role=None, addons=None, service_level=None, usage=None, jwt_token=None, **kwargs):
        # We accept a kwargs argument so that the DBus object can pass the options dictionary it
        # receives transparently to the service via dictionary unpacking.  This strategy allows the
        # DBus object to be more independent of the service implementation.

        # If there are any values in kwargs that don't map to keyword arguments defined in the message
        # signature we want to consider that an error.
        if kwargs:
            raise exceptions.ValidationError(_("Unknown arguments: %s") % kwargs.keys())

        syspurpose = syspurposelib.read_syspurpose()

        save_syspurpose = False

        # First set new syspurpose values, if there is any
        if role is not None:
            syspurpose['role'] = role
            save_syspurpose = True
        if addons is not None:
            syspurpose['addons'] = addons
            save_syspurpose = True
        if service_level is not None:
            syspurpose['service_level_agreement'] = service_level
            save_syspurpose = True
        if usage is not None:
            syspurpose['usage'] = usage
            save_syspurpose = True

        # Then try to get all syspurpose values
        role = syspurpose.get('role', '')
        addons = syspurpose.get('addons', [])
        usage = syspurpose.get('usage', '')
        service_level = syspurpose.get('service_level_agreement', '')

        type = type or "system"

        options = {
            'activation_keys': activation_keys,
            'environment': environment,
            'force': force,
            'name': name,
            'consumerid': consumerid,
            'type': type,
            'jwt_token': jwt_token
        }
        self.validate_options(options)

        environment = options['environment']
        facts_dict = self.facts.get_facts()

        # Default to the hostname if no name is given
        consumer_name = options['name'] or socket.gethostname()

        self.plugin_manager.run("pre_register_consumer", name=consumer_name, facts=facts_dict)

        if consumerid:
            consumer = self.cp.getConsumer(consumerid)
            if consumer.get('type', {}).get('manifest', {}):
                raise exceptions.ServiceError(
                    "Registration attempted with a consumer ID that is not of type 'system'")
        else:
            consumer = self.cp.registerConsumer(
                name=consumer_name,
                facts=facts_dict,
                owner=org,
                environment=environment,
                keys=options.get('activation_keys'),
                installed_products=self.installed_mgr.format_for_server(),
                content_tags=self.installed_mgr.tags,
                type=type,
                role=role,
                addons=addons,
                service_level=service_level,
                usage=usage,
                jwt_token=jwt_token
            )
        self.installed_mgr.write_cache()
        self.plugin_manager.run("post_register_consumer", consumer=consumer, facts=facts_dict)
        managerlib.persist_consumer_cert(consumer)

        # Now that we are registered, load the new identity
        self.identity.reload()

        # If new syspurpose values were given as arguments, then save these values to syspurpose.json now
        if save_syspurpose is True:
            syspurposelib.write_syspurpose(syspurpose)

        syspurpose_dict = {
            'service_level_agreement': consumer['serviceLevel'] if 'serviceLevel' in list(consumer.keys()) else '',
            'role': consumer['role'] if 'role' in list(consumer.keys()) else '',
            'usage': consumer['usage'] if 'usage' in list(consumer.keys()) else '',
            'addons': consumer['addOns'] if 'addOns' in list(consumer.keys()) else []
        }

        # Try to do three-way merge and then save result to syspurpose.json file
        local_result = syspurposelib.merge_syspurpose_values(remote=syspurpose_dict, base={})
        syspurposelib.write_syspurpose(local_result)

        # Save syspurpose attributes from consumer to cache file
        syspurposelib.write_syspurpose_cache(syspurpose_dict)

        return consumer

    def validate_options(self, options):
        if self.identity.is_valid() and options['force'] is not True:
            raise exceptions.ValidationError(_("This system is already registered. Add force to options to "
                "override."))
        elif options.get('name') == '':
            raise exceptions.ValidationError(_("Error: system name can not be empty."))
        elif options['consumerid'] and options['force'] is True:
            raise exceptions.ValidationError(_("Error: Can not force registration while attempting to "
                "recover registration with consumerid. Please use --force without --consumerid to re-register"
                " or use the clean command and try again without --force."))

        # If 'activation_keys' already exists in the dictionary, leave it.  Otherwise, set to None.
        if options['activation_keys']:
            # 746259: Don't allow the user to pass in an empty string as an activation key
            if '' == options['activation_keys']:
                raise exceptions.ValidationError(_("Error: Must specify an activation key"))
            elif getattr(self.cp, 'username', None) or getattr(self.cp, 'password', None):
                raise exceptions.ValidationError(_("Error: Activation keys do not require user credentials."))
            elif options['consumerid']:
                raise exceptions.ValidationError(_("Error: Activation keys can not be used with previously"
                    " registered IDs."))
            elif options['environment']:
                raise exceptions.ValidationError(_("Error: Activation keys do not allow environments to be"
                    " specified."))
        elif options.get('jwt_token') is not None:
            # TODO: add more checks here
            pass
        elif not getattr(self.cp, 'username', None) or not getattr(self.cp, 'password', None):
            if not getattr(self.cp, 'token', None):
                raise exceptions.ValidationError(_("Error: Missing username or password."))



from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2010 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

import datetime
import logging
import re
import threading
import socket

from subscription_manager.ga import GObject as ga_GObject
from subscription_manager.ga import Gtk as ga_Gtk
from subscription_manager.ga import gtk_compat as ga_gtk_compat

from subscription_manager.exceptions import ExceptionMapper
import rhsm.connection as connection
from subscription_manager.gui import messageWindow

log = logging.getLogger(__name__)

# we need gtk 2.18+ to do the right markup in linkify
MIN_GTK_MAJOR = 2
MIN_GTK_MINOR = 18
MIN_GTK_MICRO = 0

EVEN_ROW_COLOR = '#eeeeee'

# set if we are in firstboot, to disable linkify, see bz#814378
FIRSTBOOT = False


def running_as_firstboot():
    global FIRSTBOOT
    FIRSTBOOT = True


def get_running_as_firstboot():
    return FIRSTBOOT


def test_proxy_reachability(proxy_server, proxy_port):
    """
    Function used for testing reachability of proxy server. Note: this
    function does not test functionality of proxy server.
    :return: True, when proxy is reachable. Otherwise it returns False.
    """

    result = None
    try:
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.settimeout(10)
        result = sock.connect_ex((proxy_server, proxy_port))
    except socket.error as e:
        log.error("Attempted bad proxy: %s" % e)
    finally:
        sock.close()

    if result != 0:
        log.error("Proxy connection error: %s" % result)
        return False
    else:
        return True


def handle_gui_exception(e, msg, parent, format_msg=True, log_msg=None):
    """
    Handles an exception for the gui by logging the stack trace and
    displaying a user-friendly internationalized message.

    e = either an exception or a tuple returned from sys.exc_info()
    msg = User friendly message to display in GUI.
    parent = Parent window where the error originates.
    log_msg = Optional message to be logged in addition to stack trace.
    format_msg = if true, string sub the exception error in the msg
    """
    if isinstance(e, tuple):
        if not log_msg:
            log_msg = str(e[1])

        log.error(log_msg, exc_info=e)
        # Get the class instance of the exception
        e = e[1]
    else:
        if log_msg:
            log.error(log_msg)
        log.exception(e)

    exception_mapper = ExceptionMapper()
    mapped_message = exception_mapper.get_message(e)
    if mapped_message:
        if isinstance(e, connection.RestlibException):
            # If this exception's code is in the 200 range (such as 202 ACCEPTED)
            # we're going to ignore the message we were given and just display
            # the message from the server as an info dialog. (not an error)
            if 200 < int(e.code) < 300:
                message = linkify(mapped_message)
                messageWindow.InfoDialog(messageWindow.wrap_text(message))

            else:
                try:
                    if format_msg:
                        message = msg % linkify(mapped_message)
                    else:
                        message = linkify(mapped_message)
                except Exception:
                    message = msg

                show_error_window(message, parent=parent)
        else:
            show_error_window(mapped_message, parent)
    else:
        #catch-all, try to interpolate and if it doesn't work out, just display the message
        try:
            interpolated_str = msg % e
            show_error_window(interpolated_str, parent=parent)
        except Exception:
            show_error_window(msg, parent=parent)


def format_mapped_message(e, msg, mapped_message, format_msg=True):
    message = None
    if isinstance(e, connection.RestlibException):
        # If this exception's code is in the 200 range (such as 202 ACCEPTED)
        # we're going to ignore the message we were given and just display
        # the message from the server as an info dialog. (not an error)
        if 200 < int(e.code) < 300:
            message = linkify(mapped_message)
            return message
    try:
        if format_msg:
            message = msg % linkify(mapped_message)
        else:
            message = linkify(mapped_message)
    except Exception:
        message = msg
    return message


def format_interpolated_message(e, msg, mapped_message, format_msg=True):
    message = None
    #catch-all, try to interpolate and if it doesn't work out, just display the message
    try:
        interpolated_str = msg % e
        message = interpolated_str
    except Exception:
        message = msg
    return message


def format_exception(e, msg, format_msg=True, log_msg=None):
    if isinstance(e, tuple):
        log.error(log_msg, exc_info=e)
        # Get the class instance of the exception
        e = e[1]
    message = None
    exception_mapper = GuiExceptionMapper()
    mapped_message = exception_mapper.get_message(e)
    if mapped_message:
        message = format_mapped_message(e, msg, mapped_message, format_msg=format_msg)
    else:
        message = format_interpolated_message(e, msg, mapped_message, format_msg=format_msg)

    return message


# FIXME: This should be in messageWindow.py (or better, removed)
def show_error_window(message, parent=None):
    messageWindow.ErrorDialog(messageWindow.wrap_text(message),
                              parent)


def show_info_window(message, parent=None):
    messageWindow.InfoDialog(messageWindow.wrap_text(message),
                             parent)


def linkify(msg):
    """
    Parse a string for any urls and wrap them in a hrefs, for use in a
    gtklabel.
    """
    # http (non whitespace or . or
    #  ? or () or - or / or ;
    url_regex = re.compile("""https?://[\w\.\?\(\)\-\/]*""")

    if ga_Gtk.check_version(MIN_GTK_MAJOR, MIN_GTK_MINOR, MIN_GTK_MICRO):
        return msg

    # dont linkify in firstboot
    if FIRSTBOOT:
        return msg

    def add_markup(mo):
        url = mo.group(0)
        return '<a href="%s">%s</a>' % (url, url)

    return url_regex.sub(add_markup, msg)


def apply_highlight(text, highlight):
    """
    Apply pango markup to highlight a search term in a string
    """
    if not highlight:
        return ga_GObject.markup_escape_text(text)

    regex = re.compile("(" + re.escape(highlight) + ")", re.I)
    parts = regex.split(text)

    escaped = []
    # re.split makes every second result be our split term
    on_search_term = False
    for part in parts:
        if on_search_term:
            escaped += "<b>%s</b>" % ga_GObject.markup_escape_text(part)
        else:
            escaped += ga_GObject.markup_escape_text(part)
        on_search_term = not on_search_term

    return "".join(escaped)


def find_text(haystack, needle):
    """
    Find all occurances of needle in haystack, case insensitvely.
    Return a list of the offsets of all the occurances
    """
    if not needle:
        return []

    needle = needle.lower()
    haystack = haystack.lower()

    finds = []
    offset = 0

    while True:
        index = haystack.find(needle, offset)
        if (index == -1):
            break
        finds.append(index)
        offset = index + 1
        if (index + 1 == len(haystack)):
            break

    return finds


def make_today_none(today):
    """
    Given a datetime, return either that datetime, or, if the value is today's
    date, return None, to use the servers current time

    Useful for asking for subscriptions that are valid now.
    """
    if today.date() == datetime.date.today():
        return None
    return today


def get_cell_background_color(item_idx):
    """
    Determines the BG color for a cell based on entry index of the model.
    """
    # NOTE: Even indexes are actually displayed as an odd row.
    if item_idx % 2 != 0:
        return EVEN_ROW_COLOR


def set_background_model_index(tree_view, model_idx):
    """
    Sets the model index containing the background color for all cells.
    This should be called after all columns and renderes have been added
    to the treeview.

    @param tree_view: the tree view to set the background model index on.
    @param model_idx: the model index containing the background color.
    """
    for col in tree_view.get_columns():
        for renderer in col.get_cell_renderers():
            col.add_attribute(renderer, 'cell-background', model_idx)


def gather_group(store, iter, group):
    """
    Returns a list of TreeRowReferences for an iter and every child of the iter
    """
    if store.iter_has_child(iter):
        child_iter = store.iter_children(iter)
        while child_iter:
            gather_group(store, child_iter, group)
            child_iter = store.iter_next(child_iter)

    refs = ga_gtk_compat.tree_row_reference(store, store.get_path(iter))
    group.append(refs)

    return group


class WidgetUpdate(object):

    def __init__(self, *widgets_to_disable):
        self.widgets_to_disable = widgets_to_disable
        self.set_sensitive(False)

    def set_sensitive(self, is_sensitive):
        for widget in self.widgets_to_disable:
            widget.set_sensitive(is_sensitive)

    def finished(self):
        self.set_sensitive(True)


class AsyncWidgetUpdater(object):

    def __init__(self, parent):
        self.parent_window = parent

    def worker(self, widget_update, backend_method, args=None, kwargs=None, exception_msg=None, callback=None):
        args = args or []
        kwargs = kwargs or {}
        try:
            result = backend_method(*args, **kwargs)
            if callback:
                ga_GObject.idle_add(callback, result)
        except Exception as e:
            message = exception_msg or str(e)
            ga_GObject.idle_add(handle_gui_exception, e, message, self.parent_window)
        finally:
            ga_GObject.idle_add(widget_update.finished)

    def update(self, widget_update, backend_method, args=None, kwargs=None, exception_msg=None, callback=None):
        threading.Thread(target=self.worker, name="AsyncWidgetUpdaterThread",
                         args=(widget_update, backend_method, args,
                               kwargs, exception_msg, callback)).start()


class GuiExceptionMapper(ExceptionMapper):

    def format_restlib_exception(self, restlib_exception, message_template):
        return ga_GObject.markup_escape_text(restlib_exception.msg)



from __future__ import print_function, division, absolute_import

try:
    import unittest2 as unittest
except ImportError:
    import unittest

import os
import subprocess
import sys
import shutil
import tempfile
import threading
import time

from nose import SkipTest

from subscription_manager import lock


class TestLock(unittest.TestCase):
    lf_name = "lock.file"

    def setUp(self):
        self.other_process = None

    def _lock_path(self):
        tmp_dir = tempfile.mkdtemp(suffix="-lock", prefix="subman-unit-tests-")
        self.addCleanup(shutil.rmtree, tmp_dir, ignore_errors=True)
        return os.path.join(tmp_dir, self.lf_name)

    # For thread.Timer()
    def _kill_other_process(self, other_process):
        self.fail("nothing happened before we timed out.")
        # die die die
        other_process.terminate()
        other_process.kill()
        self.timer.cancel()

    def _grab_lock_from_other_pid(self, lockfile_path,
                                  other_process_timeout=None,
                                  acquire_timeout=None):
        # klugey
        other_process_timeout = other_process_timeout or 3.0
        acquire_timeout = acquire_timeout or 5.0

        sys_path = os.path.join(os.path.dirname(__file__), "../src")
        self.other_process = subprocess.Popen([sys.executable, __file__, lockfile_path],
                                              close_fds=True,
                                              stdin=subprocess.PIPE,
                                               env={'PYTHONPATH': sys_path})

        # make sure other process has had time to create the lock file
        while True:
            lock_exists = os.path.exists(lockfile_path)
            if lock_exists:
                break
            time.sleep(0.05)

        # in another thread, wait 3 seconds, then send 'whatever' to stdin of
        # other process so it closes. A timeout...
        def wait_for_pid(timer):
            time.sleep(other_process_timeout)
            self.close_lock_holder()
            timer.cancel()

        timer = threading.Timer(acquire_timeout, self.timeout_fail)

        op_thread = threading.Thread(target=wait_for_pid, args=[timer])
        op_thread.start()

        return op_thread

    def close_lock_holder(self):
        try:
            self.other_process.communicate("whatever".encode('utf-8'))
        except Exception as e:
            print(e)
            # whatever, we closed it in the other thread

    def timeout_fail(self):
        self.close_lock_holder()
        self.fail("timeoutsdfsdf")

    def test_two_pids_blocking_none_blocks(self):
        # This test will either fail occasionally, or have to wait an
        # unreasonable time period, which just slows down the test suite.
        # Left in code since it is a useful test if changing lock behavior,
        # but too troublesome in general.
        skip_test = True
        if skip_test:
            raise SkipTest('Skipping lock.test_two_pids_blocking_none_blocks')

        lock_path = self._lock_path()
        # start a different proc that holds the lock, that times out after 3
        self._grab_lock_from_other_pid(lock_path, 1.0, 0.2)

        b = lock.Lock(lock_path)

        res = b.acquire()
        self.assertTrue(res is None)

    def test_two_pids_blocking_none(self):
        lock_path = self._lock_path()
        # start a different proc that holds the lock, that times out after 3
        self._grab_lock_from_other_pid(lock_path, 0.2, 1.0)

        b = lock.Lock(lock_path)
        res = b.acquire()
        self.assertTrue(b.acquired())
        self.assertTrue(res is None)

    def test_two_pids_blocking_true(self):
        lock_path = self._lock_path()
        # start a different proc that holds the lock, that times out after 3
        self._grab_lock_from_other_pid(lock_path, 0.2, 1.0)
        b = lock.Lock(lock_path)
        res = b.acquire(blocking=True)
        self.assertTrue(b.acquired())
        self.assertTrue(res)

    def test_two_pids_blocking_false(self):
        lock_path = self._lock_path()
        self._grab_lock_from_other_pid(lock_path, 0.2, 1.0)
        b = lock.Lock(lock_path)
        res = b.acquire(blocking=False)
        self.assertFalse(b.acquired())
        self.other_process.communicate("whatever".encode('utf-8'))
        self.assertFalse(res)

    def test_lock(self):
        lock_path = self._lock_path()
        lf = lock.Lock(lock_path)
        self.assertEqual(lf.path, lock_path)
        self.assertEqual(lf.depth, 0)

    def test_lock_acquire(self):
        lock_path = self._lock_path()
        lf = lock.Lock(lock_path)
        res = lf.acquire()
        # given no args, acquire() blocks or returns None
        self.assertEqual(res, None)

    def test_lock_acquire_blocking_true(self):
        lock_path = self._lock_path()
        lf = lock.Lock(lock_path)
        res = lf.acquire(blocking=True)
        # acquire(blocking=True) will block or return True
        self.assertTrue(res)

    def test_lock_acquire_blocking_false(self):
        lock_path = self._lock_path()
        lf = lock.Lock(lock_path)
        res = lf.acquire(blocking=False)

        # res of False indicates lock could not be acquired without blocking
        # True indicates lock was acquired
        self.assertTrue(res)

    def test_lock_release(self):
        lock_path = self._lock_path()
        lf = lock.Lock(lock_path)
        lf.acquire()
        lf.release()

    def _stale_lock(self):
        lock_path = self._lock_path()
        fakepid = 123456789
        f = open(lock_path, 'w')
        f.write('%s\n' % fakepid)
        f.close()
        return lock_path

    def test_lock_acquire_stale_pid(self):
        lock_path = self._stale_lock()
        lf = lock.Lock(lock_path)
        res = lf.acquire(blocking=True)
        self.assertTrue(res)

    def test_lock_acquire_stale_pid_nonblocking(self):
        lock_path = self._stale_lock()
        lf = lock.Lock(lock_path)
        res = lf.acquire(blocking=False)
        self.assertTrue(res)

# always blocks, needs eventloop/threads
#    def test_lock_drive_full_blocking(self):
#        lock_path = "/dev/full"
#        lf = lock.Lock(lock_path)
#        res = lf.acquire(blocking=True)
#        log.debug(res)

# FIXME: the lockfile creation fails on /dev/full
#    def test_lock_drive_full_nonblocking(self):
#        lock_path = "/dev/full"
#        lf = lock.Lock(lock_path)
#        res = lf.acquire(blocking=False)
#        self.assertFalse(res)


# run this module's main in a subprocess to grab a lock from a different
# pid.
def main(args):
    lock_file_path = args[1]
    test_lock = lock.Lock(lock_file_path)

    # could return a useful value, so the thread communicating with
    # it could notice it couldn't get the lock
    res = test_lock.acquire(blocking=False)
    if res is False:
        return 128

    # exit on any stdin input
    for line in sys.stdin.readlines():
        return 1


if __name__ == "__main__":
    sys.exit(main(sys.argv[:]))



from __future__ import print_function, division, absolute_import

import logging

import mock
import tempfile
import os

from . import fixture

from . import stubs

from rhsm import logutil


# no NullHandler in 2.6, include our own
class NullHandler(logging.Handler):
    def emit(self, record):
        pass


class TestLogutil(fixture.SubManFixture):
    def setUp(self):
        super(TestLogutil, self).setUp()
        self.rhsm_config = stubs.StubConfig()
        rhsm_patcher = mock.patch('rhsm.config')
        self.rhsm_config_mock = rhsm_patcher.start()
        self.rhsm_config_mock.get_config_parser.return_value = self.rhsm_config
        self.addCleanup(rhsm_patcher.stop)

    def tearDown(self):
        self.remove_loggers()
        super(TestLogutil, self).tearDown()

    def remove_loggers(self):
        logging.getLogger("subscription_manager").handlers = []
        logging.getLogger("rhsm").handlers = []
        logging.getLogger("rhsm-app").handlers = []
        logging.getLogger().handlers = []

    def test_log_init(self):
        logutil.init_logger()
        sm_logger = logging.getLogger("subscription_manager")
        rhsm_logger = logging.getLogger("rhsm-app")
        sm_effective = sm_logger.getEffectiveLevel()
        rhsm_effective = rhsm_logger.getEffectiveLevel()
        # Fun hack for 2.6/2.7 interoperability
        self.assertTrue(
            logging.DEBUG == sm_effective or
            logging._levelNames[sm_effective] == logging.DEBUG)
        self.assertTrue(
            logging.DEBUG == rhsm_effective or
            logging._levelNames[rhsm_effective] == logging.DEBUG)

    def test_log_init_default_log_level(self):
        self.rhsm_config.set("logging", "default_log_level", "WARNING")

        logutil.init_logger()
        sm_logger = logging.getLogger("subscription_manager")
        rhsm_logger = logging.getLogger("rhsm-app")
        sm_effective = sm_logger.getEffectiveLevel()
        rhsm_effective = rhsm_logger.getEffectiveLevel()
        # Fun hack for 2.6/2.7 interoperability
        self.assertTrue(
            logging.WARNING == sm_effective or
            logging._levelNames[sm_effective] == logging.WARNING)
        self.assertTrue(
            logging.WARNING == rhsm_effective or
            logging._levelNames[rhsm_effective] == logging.WARNING)

    def test_init_logger_for_yum(self):
        logutil.init_logger_for_yum()
        sm_logger = logging.getLogger("subscription_manager")
        rhsm_logger = logging.getLogger("rhsm-app")
        self.assertFalse(sm_logger.propagate)
        self.assertFalse(rhsm_logger.propagate)

    def test_do_not_modify_root_logger(self):
        root_handlers = logging.getLogger().handlers
        logutil.init_logger()
        self.assert_items_equals(logging.getLogger().handlers, root_handlers)

    def test_set_valid_logger_level(self):
        logging_conf = [
            ('subscription_manager.managercli', "ERROR"),
            ('rhsm', "WARNING"),
            ('rhsm-app', "CRITICAL")
        ]

        for logger_name, log_level in logging_conf:
            self.rhsm_config.set('logging', logger_name, log_level)

        logutil.init_logger()

        for logger_name, log_level in logging_conf:
            real_log_level = logging.getLogger(logger_name).getEffectiveLevel()
            self.assertTrue(
                logging.getLevelName(log_level) == real_log_level or
                log_level == real_log_level)

    def test_set_invalid_logger_level(self):
        test_logger_name = 'foobar'
        initial_level = logging.ERROR
        test_logger = logging.getLogger(test_logger_name)
        test_logger.setLevel(initial_level)
        config_level = logging.DEBUG
        self.rhsm_config.set('logging', test_logger_name,
                             config_level)

        logutil.init_logger()

        self.assertNotEqual(test_logger.getEffectiveLevel(), config_level)

    def test_missing_log_directory(self):
        """
        Test creating directory for log directory
        """
        temp_dir_path = tempfile.mkdtemp()

        old_dir_path = logutil.LOGFILE_DIR
        old_file_path = logutil.LOGFILE_PATH

        # Set logutil to uninitialized state
        logutil._rhsm_log_handler = None
        logutil._subman_debug_handler = None
        logutil.LOGFILE_DIR = temp_dir_path
        logutil.LOGFILE_PATH = os.path.join(temp_dir_path, "rhsm.log")

        self.assertTrue(os.path.exists(temp_dir_path))

        # Simulate deleting of directory /var/log/rhsm
        os.rmdir(temp_dir_path)

        logutil.init_logger()

        # init_logger should automatically create directory /var/log/rhsm,
        # when it doesn't exist
        self.assertTrue(os.path.exists(temp_dir_path))

        os.remove(logutil.LOGFILE_PATH)
        os.rmdir(logutil.LOGFILE_DIR)

        logutil.LOGFILE_DIR = old_dir_path
        logutil.LOGFILE_PATH = old_file_path

    def test_not_possible_to_create_log_dir_due_to_access_perm(self):
        """
        Test that it is not possible to create log directory due to access permission
        """
        temp_dir_path = tempfile.mkdtemp()
        os.chmod(temp_dir_path, 444)

        old_dir_path = logutil.LOGFILE_DIR
        old_file_path = logutil.LOGFILE_PATH

        # Set logutil to uninitialized state
        logutil._rhsm_log_handler = None
        logutil._subman_debug_handler = None
        logutil.LOGFILE_DIR = os.path.join(temp_dir_path, "rhsm")
        logutil.LOGFILE_PATH = os.path.join(logutil.LOGFILE_DIR, "rhsm.log")

        self.assertTrue(os.path.exists(temp_dir_path))

        with fixture.Capture() as cap:
            logutil.init_logger()

        self.assertTrue("Further logging output will be written to stderr" in cap.err)

        # init_logger should not be able to automatically create directory /var/log/rhsm,
        # when user does not have access permission for that
        self.assertFalse(os.path.exists(logutil.LOGFILE_DIR))

        os.chmod(temp_dir_path, 744)
        os.rmdir(temp_dir_path)

        logutil.LOGFILE_DIR = old_dir_path
        logutil.LOGFILE_PATH = old_file_path

    def test_wrong_rhsm_log_priv(self):
        """
        Test that error messages are not printed to stderr, when it is not possible
        to print error messages to rhsm.log during initialization of logger
        """
        # Create temporary log directory
        temp_dir_path = tempfile.mkdtemp()
        # Create temporary log file
        temp_log_file = os.path.join(temp_dir_path, "rhsm.log")
        with open(temp_log_file, "w") as fp:
            fp.write("")
        # Change permission to directory /var/log/rhsm and log file
        os.chmod(temp_log_file, 444)
        os.chmod(temp_dir_path, 444)

        old_dir_path = logutil.LOGFILE_DIR
        old_file_path = logutil.LOGFILE_PATH

        # Set logutil to uninitialized state
        logutil._rhsm_log_handler = None
        logutil._subman_debug_handler = None
        logutil.LOGFILE_DIR = temp_dir_path
        logutil.LOGFILE_PATH = os.path.join(temp_log_file)

        with fixture.Capture() as cap:
            logutil.init_logger()

        self.assertTrue("Further logging output will be written to stderr" in cap.err)

        os.chmod(temp_dir_path, 744)
        os.chmod(temp_log_file, 744)
        os.remove(logutil.LOGFILE_PATH)
        os.rmdir(logutil.LOGFILE_DIR)

        logutil.LOGFILE_DIR = old_dir_path
        logutil.LOGFILE_PATH = old_file_path



# -*- coding: utf-8 -*-

from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2018 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

"""
This module is an interface to syspurpose's SyncedStore class from subscription-manager.
It contains methods for accessing/manipulating the local syspurpose.json metadata file through SyncedStore.
"""

from rhsm.connection import ConnectionException
from subscription_manager import certlib
from subscription_manager import injection as inj

import logging
import json
import os
log = logging.getLogger(__name__)

try:
    from syspurpose.sync import sync
except ImportError:
    def sync(uep, consumer_uuid, command=None, report=None):
        log.debug("Syspurpose module unavailable, not syncing")
        return read_syspurpose()

try:
    from syspurpose.files import SyncedStore, USER_SYSPURPOSE, post_process_received_data, CACHED_SYSPURPOSE
except ImportError:
    log.debug("Could not import from module syspurpose.")
    SyncedStore = None
    USER_SYSPURPOSE = "/etc/rhsm/syspurpose/syspurpose.json"
    CACHED_SYSPURPOSE = "/var/lib/rhsm/cache/syspurpose.json"

    def post_process_received_data(data):
        return data

store = None
syspurpose = None


def save_sla_to_syspurpose_metadata(uep, consumer_uuid, service_level):
    """
    Saves the provided service-level value to the local Syspurpose Metadata (syspurpose.json) file.
    If the service level provided is null or empty, the sla value to the local syspurpose file is set to null.
    when uep and consumer_uuid is not None, then service_level is also synced with candlepin server

    :param uep: The object with uep connection (connection to candlepin server)
    :param consumer_uuid: Consumer UUID
    :param service_level: The service-level value to be saved in the syspurpose file.
    :type service_level: str
    """

    if 'SyncedStore' in globals() and SyncedStore is not None:
        synced_store = SyncedStore(uep=uep, consumer_uuid=consumer_uuid)

        # if empty, set it to null
        if service_level is None or service_level == "":
            service_level = None

        synced_store.set("service_level_agreement", service_level)
        synced_store.finish()
        log.debug("Syspurpose SLA value successfully saved locally.")
    else:
        log.error("SyspurposeStore could not be imported. Syspurpose SLA value not saved locally.")


def get_sys_purpose_store():
    """
    :return: Returns a singleton instance of the syspurpose store if it was imported.
             Otherwise None.
    """
    global store
    if store is not None:
        return store
    elif SyncedStore is not None:
        uep = inj.require(inj.CP_PROVIDER).get_consumer_auth_cp()
        uuid = inj.require(inj.IDENTITY).uuid
        store = SyncedStore(uep, consumer_uuid=uuid)
    return store


def read_syspurpose(raise_on_error=False):
    """
    Reads the system purpose from the correct location on the file system.
    Makes an attempt to use a SyspurposeStore if available falls back to reading the json directly.
    :return: A dictionary containing the total syspurpose.
    """
    if SyncedStore is not None:
        try:
            syspurpose = SyncedStore(None).get_local_contents()
        except (OSError, IOError):
            syspurpose = {}
    else:
        try:
            syspurpose = json.load(open(USER_SYSPURPOSE))
        except (os.error, ValueError, IOError):
            # In the event this file could not be read treat it as empty
            if raise_on_error:
                raise
            syspurpose = {}
    return syspurpose


def write_syspurpose(values):
    """
    Write the syspurpose to the file system.
    :param values:
    :return:
    """
    if SyncedStore is not None:
        sp = SyncedStore(None)
        sp.update_local(values)
    else:
        # Simple backup in case the syspurpose tooling is not installed.
        try:
            json.dump(values, open(USER_SYSPURPOSE), ensure_ascii=True, indent=2)
        except OSError:
            log.warning('Could not write syspurpose to %s' % USER_SYSPURPOSE)
            return False
    return True


def write_syspurpose_cache(values):
    """
    Write to the syspurpose cache on the file system.
    :param values:
    :return:
    """
    try:
        json.dump(values, open(CACHED_SYSPURPOSE, 'w'), ensure_ascii=True, indent=2)
    except OSError:
        log.warning('Could not write to syspurpose cache %s' % CACHED_SYSPURPOSE)
        return False
    return True


def get_syspurpose_valid_fields(uep=None, identity=None):
    """
    Try to get valid syspurpose fields provided by candlepin server
    :param uep: connection of candlepin server
    :param identity: current identity of registered system
    :return: dictionary with valid fields
    """
    valid_fields = {}
    cache = inj.require(inj.SYSPURPOSE_VALID_FIELDS_CACHE)
    syspurpose_valid_fields = cache.read_data(uep, identity)
    if 'systemPurposeAttributes' in syspurpose_valid_fields:
        valid_fields = syspurpose_valid_fields['systemPurposeAttributes']
    return valid_fields


def merge_syspurpose_values(local=None, remote=None, base=None):
    """
    Try to do three-way merge of local, remote and base dictionaries.
    Note: when remote is None, then this method will call REST API.
    :param local: dictionary with local values
    :param remote: dictionary with remote values
    :param base: dictionary with cached values
    :return: Dictionary with local result
    """

    if SyncedStore is None:
        return {}

    synced_store = SyncedStore(uep=None)

    if local is None:
        local = synced_store.get_local_contents()
    if remote is None:
        remote = synced_store.get_remote_contents()
    if base is None:
        base = synced_store.get_cached_contents()

    result = synced_store.merge(
        local=local,
        remote=remote,
        base=base
    )
    local_result = {key: result[key] for key in result if result[key]}
    log.debug('local result: %s ' % local_result)
    return local_result


class SyspurposeSyncActionInvoker(certlib.BaseActionInvoker):
    """
    Used by rhsmcertd to sync the syspurpose values locally with those from the Server.
    """

    def _do_update(self):
        action = SyspurposeSyncActionCommand()
        return action.perform()


class SyspurposeSyncActionReport(certlib.ActionReport):
    name = "Syspurpose Sync"

    def record_change(self, change):
        """
        Records the change detected by the three_way_merge function into a record in the report.
        :param change: A util.DiffChange object containing the recorded changes.
        :return: None
        """
        if change.source == 'remote':
            source = 'Entitlement Server'
        elif change.source == 'local':
            source = USER_SYSPURPOSE
        else:
            source = 'cached system purpose values'
        msg = None
        if change.in_base and not change.in_result:
            msg = "'{key}' removed by change from {source}".format(key=change.key,
                                                                     source=source)
        elif not change.in_base and change.in_result:
            msg = "'{key}' added with value '{value}' from change in {source}".format(
                    key=change.key, value=change.new_value, source=source
            )
        elif change.in_base and change.previous_value != change.new_value:
            msg = "'{key}' updated from '{old_value}' to '{new_value}' due to change in {source}"\
                .format(key=change.key, new_value=change.new_value,
                        old_value=change.previous_value, source=source)

        if msg:
            self._updates.append(msg)

    """
    The base method formatting does not fit the massages we are seeing here
    BZ #1789457
    """
    def format_exceptions(self):
        buf = ''
        for e in self._exceptions:
            buf += str(e).strip()
            buf += '\n'
        return buf


class SyspurposeSyncActionCommand(object):
    """
    Sync the system purpose values, by performing a three-way merge between:
      - The last known shared state (SyspurposeCache)
      - The current values on the server
      - The current values on the file system
    """

    def __init__(self):
        self.report = SyspurposeSyncActionReport()
        self.cp_provider = inj.require(inj.CP_PROVIDER)
        self.uep = self.cp_provider.get_consumer_auth_cp()

    def perform(self, include_result=False):
        """
        Perform the action that this Command represents.
        :return:
        """
        result = {}
        consumer_uuid = inj.require(inj.IDENTITY).uuid

        try:
            store = SyncedStore(
                uep=self.uep,
                consumer_uuid=consumer_uuid,
                report=self.report,
                on_changed=self.report.record_change
            )
            result = store.sync()
        except ConnectionException as e:
            self.report._exceptions.append('Unable to sync syspurpose with server: %s' % str(e))
            self.report._status = 'Failed to sync system purpose'
        self.report._updates = "\n\t\t ".join(self.report._updates)
        log.debug("Syspurpose updated: %s" % self.report)
        if not include_result:
            return self.report
        else:
            return self.report, result



from __future__ import print_function, division, absolute_import

# Copyright (c) 2011 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

import datetime

from test.fixture import SubManFixture
from test.stubs import StubUEP, StubEntitlementCertificate, \
        StubCertificateDirectory, StubProduct, StubBackend, \
        StubProductDirectory
from subscription_manager.gui.mysubstab import MySubscriptionsTab, \
        EXPIRING_IMG, WARNING_IMG, EXPIRED_IMG
from mock import Mock
from nose.plugins.attrib import attr


@attr('gui')
class MySubscriptionsTabTest(SubManFixture):

    def setUp(self):
        super(MySubscriptionsTabTest, self).setUp()
        self.uep = StubUEP
        self.backend = StubBackend()

        self.cert1 = StubEntitlementCertificate(
            StubProduct('product2'),
            start_date=datetime.datetime(2010, 1, 1),
            end_date=datetime.datetime(2060, 1, 1),
            quantity="10", ent_id='prod2')
        self.cert2 = StubEntitlementCertificate(
            StubProduct('product3'),
            start_date=datetime.datetime(2010, 1, 1),
            end_date=datetime.datetime(2060, 1, 1),
            quantity="10", ent_id='prod3')

        self.cert_dir = StubCertificateDirectory([self.cert1, self.cert2])
        self.my_subs_tab = MySubscriptionsTab(self.backend,
                                              None,
                                              self.cert_dir,
                                              StubProductDirectory([]))

    def tearDown(self):
        pass

    def test_get_entry_image_expired(self):
        cert = StubEntitlementCertificate(
                    StubProduct('product2'),
                    start_date=datetime.datetime(2010, 1, 1),
                    end_date=datetime.datetime(2011, 1, 1),
                    quantity="10", stacking_id=None)
        image = self.my_subs_tab._get_entry_image(cert)
        self.assertEqual(EXPIRED_IMG, image)

    def test_get_entry_image_expiring(self):
        tomorrow = datetime.datetime.now() + datetime.timedelta(days=1)
        cert = StubEntitlementCertificate(
                    StubProduct('product2'),
                    start_date=datetime.datetime(2010, 1, 1),
                    end_date=tomorrow,
                    quantity="10", stacking_id=None)
        image = self.my_subs_tab._get_entry_image(cert)
        self.assertEqual(EXPIRING_IMG, image)

    def test_get_entry_image_warning(self):
        ending = datetime.datetime.now() + datetime.timedelta(days=300)
        cert = StubEntitlementCertificate(
                StubProduct('product2'),
                start_date=datetime.datetime(2010, 1, 1),
                end_date=ending,
                quantity="10", ent_id='ent')
        self.my_subs_tab.backend.cs.reasons.get_subscription_reasons = Mock(return_value=['Some detail'])
        image = self.my_subs_tab._get_entry_image(cert)
        self.assertEqual(WARNING_IMG, image)

    def test_image_rank_both_none(self):
        self.assertFalse(self.my_subs_tab.image_ranks_higher(None, None))

    def test_image_rank_new_image_none(self):
        self.assertFalse(self.my_subs_tab.image_ranks_higher(EXPIRING_IMG, None))

    def test_image_rank_new_image_lower(self):
        self.assertFalse(self.my_subs_tab.image_ranks_higher(EXPIRED_IMG, EXPIRING_IMG))

    def test_image_rank_new_image_higher(self):
        self.assertTrue(self.my_subs_tab.image_ranks_higher(EXPIRING_IMG, EXPIRED_IMG))

    def test_image_rank_old_image_none(self):
        self.assertTrue(self.my_subs_tab.image_ranks_higher(None, EXPIRED_IMG))

    def test_image_rank_warn_none(self):
        self.assertTrue(self.my_subs_tab.image_ranks_higher(None, WARNING_IMG))

    def test_image_rank_warn_expiring(self):
        self.assertTrue(self.my_subs_tab.image_ranks_higher(WARNING_IMG, EXPIRING_IMG))

    def test_image_rank_warn_expired(self):
        self.assertTrue(self.my_subs_tab.image_ranks_higher(WARNING_IMG, EXPIRED_IMG))

    def test_correct_cert_data_inserted_into_store(self):
        self.cert1.order.stacking_id = None
        self.cert2.order.stacking_id = None

        column_entries = self._get_entries_for_test()

        self.assertEqual(2, len(column_entries))

        self._assert_entry_1(column_entries[0])
        self._assert_entry_2(column_entries[1])

    def test_stacking_entry_not_inserted_when_stacking_id_exists(self):
        self.cert1.order.stacking_id = 1234
        self.cert2.order.stacking_id = None
        column_entries = self._get_entries_for_test()

        # single entry with stacking_id: no stacking entry
        self.assertEqual(2, len(column_entries))

        self._assert_entry_1(column_entries[0])
        self._assert_entry_2(column_entries[1])

    def test_stacking_entry_inserted_when_stacking_id_exists(self):
        self.cert1.order.stacking_id = 1234
        self.cert2.order.stacking_id = 1234
        column_entries = self._get_entries_for_test()

        self.assertEqual(3, len(column_entries))

        self._assert_group_entry(column_entries[0])
        self._assert_entry_1(column_entries[1])
        self._assert_entry_2(column_entries[2])

    def test_no_subscriptions_unregister_button_is_blank(self):
        cert_dir = StubCertificateDirectory([])
        my_subs_tab = MySubscriptionsTab(self.backend,
                                         None,
                                         cert_dir,
                                         StubProductDirectory([]))
        self.assertFalse(my_subs_tab.unsubscribe_button.get_property('sensitive'))

    def test_unselect_unregister_button_is_blank(self):
        self.my_subs_tab.on_no_selection()
        self.assertFalse(self.my_subs_tab.unsubscribe_button.get_property('sensitive'))

    def _get_entries_for_test(self):
        column_entries = []

        def collect_entries(tree_iter, entry):
            column_entries.append(entry)

        # Test that the data from a subscription is loaded into the store.
        self.my_subs_tab.store.add_map = collect_entries
        self.my_subs_tab.update_subscriptions()
        return column_entries

    def _assert_entry_1(self, entry):
        self.assertEqual(self.cert1.order.name, entry['subscription'])
        self.assertEqual(self.cert1.valid_range.begin(), entry['start_date'])
        self.assertEqual(self.cert1.valid_range.end(), entry['expiration_date'])
        self.assertEqual("0 / 1", entry['installed_text'])
        self.assertEqual(0, entry['installed_value'])
        # The quantity/serial column is of type string, so when we fetch it from the
        # widget, it is a str.
        self.assertEqual(str(self.cert1.order.quantity_used), entry['quantity'])
        self.assertEqual(str(self.cert1.serial), entry['serial'])
        self.assertFalse(entry['is_group_row'])

    def _assert_entry_2(self, entry):
        self.assertEqual(self.cert2.order.name, entry['subscription'])
        self.assertEqual(self.cert2.valid_range.begin(), entry['start_date'])
        self.assertEqual(self.cert2.valid_range.end(), entry['expiration_date'])
        self.assertEqual("0 / 1", entry['installed_text'])
        self.assertEqual(0, entry['installed_value'])
        self.assertEqual(str(self.cert2.order.quantity_used), entry['quantity'])
        self.assertEqual(str(self.cert2.serial), entry['serial'])
        self.assertFalse(entry['is_group_row'])

    def _assert_group_entry(self, entry):
        self.assertEqual("Stack of %s and 1 other" % self.cert1.order.name,
                          entry['subscription'])
        self.assertFalse('start_date' in entry)
        self.assertFalse('expiration_date' in entry)
        self.assertFalse('installed_text' in entry)
        self.assertEqual(0.0, entry['installed_value'])
        self.assertFalse('quantity' in entry)
        self.assertFalse('serial' in entry)
        self.assertTrue(entry['is_group_row'])



from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2010 Red Hat, Inc.
#
# Authors: Jeff Ortel <jortel@redhat.com>
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
from gzip import GzipFile
import logging
import os
import six
# for labelCompare
import rpm

from rhsm.certificate import create_from_pem

from subscription_manager.certdirectory import Directory, DEFAULT_PRODUCT_CERT_DIR
from subscription_manager.injection import PLUGIN_MANAGER, require

from subscription_manager import utils
from subscription_manager import repolib

import subscription_manager.injection as inj
from rhsm import ourjson as json

log = logging.getLogger(__name__)


class DatabaseDirectory(Directory):

    PATH = 'var/lib/rhsm'

    def __init__(self):
        super(DatabaseDirectory, self).__init__(self.PATH)
        self.create()


class ProductIdRepoMap(utils.DefaultDict):

    def __init__(self, *args, **kwargs):
        self.default_factory = list


class ProductDatabase(object):

    def __init__(self):
        self.dir = DatabaseDirectory()
        self.content = ProductIdRepoMap()
        self.create()

    def add(self, product, repo):
        self.content[product].append(repo)

    # TODO: need way to delete one prod->repo map
    def delete(self, product):
        try:
            del self.content[product]
        except Exception:
            pass

    def find_repos(self, product):
        return self.content.get(product, None)

    def create(self):
        if not os.path.exists(self.__fn()):
            self.write()

    def read(self):
        f = open(self.__fn())
        try:
            d = json.load(f)
            # munge old format to new if need be
            self.populate_content(d)
        except Exception:
            pass
        f.close()

    def populate_content(self, db_dict):
        """Populate map with info from a productid -> [repoids] map.

        Note this needs to support the old form of
        a {"productid": "repoid"} as well as the
        new form of {"productid: ["repoid1",...]}"""
        for productid, repo_data in list(db_dict.items()):
            if isinstance(repo_data, six.string_types):
                self.content[productid].append(repo_data)
            else:
                self.content[productid] = repo_data

    def write(self):
        f = open(self.__fn(), 'w')
        try:
            json.dump(self.content, f, indent=2, default=json.encode)
        except Exception:
            pass
        f.close()

    def __fn(self):
        return self.dir.abspath('productid.js')


class ComparableMixin(object):
    """Needs compare_keys to be implemented."""
    def _compare(self, keys, method):
        return method(keys[0], keys[1]) if keys else NotImplemented

    def __eq__(self, other):
        return self._compare(self.compare_keys(other), lambda s, o: s == o)

    def __ne__(self, other):
        return self._compare(self.compare_keys(other), lambda s, o: s != o)

    def __lt__(self, other):
        return self._compare(self.compare_keys(other), lambda s, o: s < o)

    def __gt__(self, other):
        return self._compare(self.compare_keys(other), lambda s, o: s > o)

    def __le__(self, other):
        return self._compare(self.compare_keys(other), lambda s, o: s <= o)

    def __ge__(self, other):
        return self._compare(self.compare_keys(other), lambda s, o: s >= o)


class RpmVersion(object):
    """Represent the epoch, version, release of a rpm style version.

    This includes the rich comparison methods to support >,<,==,!-
    using rpm's labelCompare.

    See http://fedoraproject.org/wiki/Archive:Tools/RPM/VersionComparison
    for more details of the actual comparison rules.
    """

    # Ordered list of suffixes
    suffixes = ['alpha', 'beta']

    def __init__(self, epoch="0", version="0", release="1"):
        self.epoch = epoch
        self.version = version
        self.release = release

    @property
    def evr(self):
        return (self.epoch, self.version, self.release)

    @property
    def evr_nosuff(self):
        def no_suff(s):
            for suff in self.suffixes:
                if s and s.lower().endswith(suff):
                    return s[:-len(suff)].strip('- ')
            return s
        return (self.epoch, no_suff(self.version), self.release)

    def compare(self, other):
        def ends_with_which(s):
            for idx, suff in enumerate(self.suffixes):
                if s.lower().endswith(suff):
                    return idx
            # Easier compare
            return len(self.suffixes)

        raw_compare = rpm.labelCompare(self.evr, other.evr)
        non_beta_compare = rpm.labelCompare(self.evr_nosuff, other.evr_nosuff)
        if non_beta_compare != raw_compare:
            if ends_with_which(self.version) < ends_with_which(other.version):
                return -1
            return 1
        return raw_compare

    def __lt__(self, other):
        lc = self.compare(other)
        if lc == -1:
            return True
        return False

    def __le__(self, other):
        lc = self.compare(other)
        if lc > 0:
            return False
        return True

    def __eq__(self, other):
        lc = self.compare(other)
        if lc == 0:
            return True
        return False

    def __ne__(self, other):
        lc = self.compare(other)
        if lc != 0:
            return True
        return False


class ComparableProduct(ComparableMixin):
    """A comparable version from a Product. For comparing and sorting Product objects.

    Products are never equals if they do not have the same product id.
    lt and gt for different products are also always false.

    NOTE: This object doesn't make sense to compare Products with different
    Product ID. The results are kind of nonsense for that case.

    This could be extended to compare, either with a more complicated
    version compare, or using other attributes.

    Awesomeos-1.1 > Awesomeos-1.0
    Awesomeos-1.1 != Awesomeos-1.0
    Awesomeos-1.0 < Awesomeos-1.0

    The algorithm used for comparisions is the rpm version compare, as used
    by rpm, yum, etc. Also known as "rpmvercmp" or "labelCompare".

    There aren't any standard product version comparison rules, but the
    rpm rules are pretty flexible, documented, and well understood.
    """
    def __init__(self, product):
        self.product = product

    def compare_keys(self, other):
        """Create a a tuple of RpmVersion objects.

        Create a RpmVersion using the product's version attribute
        as the 'version' attribute for a rpm label tuple. We let the
        epoch default to 0, and the release to 1 for each, so we are
        only comparing the difference in the version attribute.
        """
        if self.product.id == other.product.id:
            return (RpmVersion(version=self.product.version),
                    RpmVersion(version=other.product.version))
        return None

    def __str__(self):
        return "<ComparableProduct id=%s version=%s name=%s product=%s>" % \
                (self.product.id, self.product.version, self.product.name, self.product)


class ComparableProductCert(ComparableMixin):
    """Compareable version of ProductCert.

    Used to determine the "newer" of two ProductCerts. Initially just based
    on a comparison of a ComparableProduct built from the Product, which compares
    using the Product.version field."""

    def __init__(self, product_cert):
        self.product_cert = product_cert
        self.product = self.product_cert.products[0]
        self.comp_product = ComparableProduct(self.product)

    # keys used to compare certificate. For now, just the keys for the
    # Product.version. This could include say, certificate serial or issue date
    def compare_keys(self, other):
        return self.comp_product.compare_keys(other.comp_product)


class ProductId(object):
    def __init__(self, product_cert):
        self.product_cert = product_cert

    # write out the cert
    def install(self):
        """Write out the product cert and run anything
        to trigger based on that"""

        pass

    def remove(self):
        """Delete product cert from the filesystem.

        Subclasses should override this."""
        pass

    # def compare(self, other):   # version check?


class ProductManager(object):
    """Manager product certs, detecting when they need to be installed, or deleted.

    Note that this class has no knowledge of when it runs, and no nothing of the
    rpm transaction that may have causes it to run. So it only looks at the state
    of installed packages, yum repo states, installed product certs, and the
    product id->repo id mapping db productid.js.

    It finds yum repo's which are enabled.
    It finds repo's which are active. "active" in this case means one or more
      installed packages were installed from that repo. It does this my checking
      the 'repoid' field yum reports for each installed package.

    It removes certs that are no longer needed. If no packages are installed from
      a product (and more specifically, the repo's created from that product), it
      is considered unneeded and removed.

    Args:
        product_dir: a ProductDirectory class (optional)
        product_db: A ProductDatabase class (optional)
    """

    PRODUCTID = 'productid'

    def __init__(self, product_dir=None, product_db=None):

        self.pdir = product_dir
        if not product_dir:
            self.pdir = inj.require(inj.PROD_DIR)

        self.db = product_db
        if not product_db:
            self.db = ProductDatabase()

        self.db.read()
        self.meta_data_errors = []

        self.plugin_manager = require(PLUGIN_MANAGER)

    def find_disabled_repos(self):
        """Find repos disabled in redhat.repo"""
        repo_file = repolib.YumRepoFile()
        repo_file.read()

        disabled_in_redhat_repo = []
        for section in repo_file.sections():
            repo = repo_file.section(section)

            if not utils.is_true_value(repo.get('enabled', '0')):
                disabled_in_redhat_repo.append(repo.id)

        return disabled_in_redhat_repo

    def find_temp_disabled_repos(self, enabled):
        """Find repo from redhat.repo that have been disabled from cli."""
        yum_enabled = [x[1] for x in enabled]

        # Read the redhat.repo file so we can check if any of our
        # repos have been disabled by --disablerepo or another plugin.
        repo_file = repolib.YumRepoFile()
        repo_file.read()

        enabled_in_redhat_repo = []
        for section in repo_file.sections():
            repo = repo_file.section(section)

            if utils.is_true_value(repo.get('enabled', '0')):
                enabled_in_redhat_repo.append(repo.id)

        temp_disabled = []
        for enabled_repo in enabled_in_redhat_repo:
            if enabled_repo not in yum_enabled:
                temp_disabled.append(enabled_repo)

        return temp_disabled

    def update(self, enabled, active, tracks_repos):
        # populate the temp_disabled list so update_remove has it
        # this could likely happen later...
        temp_disabled_repos = self.find_temp_disabled_repos(enabled)

        # only execute this on versions of yum that track
        # which repo a package came from, aka, 3.2.28 and newer
        if tracks_repos:
            # check that we have any repo's enabled
            # and that we have some enabled repo's. Not just
            # that we have packages from repo's that are
            # not active. See #806457
            if enabled and (active or temp_disabled_repos):
                # Check that we have either active repos
                # or that we have temp_disabled_repos
                # See bz 1222627
                self.update_removed(active, temp_disabled_repos)

        # TODO: it would probably be useful to keep track of
        # the state a bit, so we can report what we did
        self.update_installed(enabled, active)

    def _is_workstation(self, product_cert):
        if product_cert.name == "Red Hat Enterprise Linux Workstation" and \
                "rhel-5-client-workstation" in product_cert.provided_tags and \
                product_cert.version[0] == '5':
            return True
        return False

    def _is_desktop(self, product_cert):
        if product_cert.name == "Red Hat Enterprise Linux Desktop" and \
                "rhel-5-client" in product_cert.provided_tags and \
                product_cert.version[0] == '5':
            return True
        return False

    def update_installed(self, enabled, active):
        """Install product certs for products with enabled and active repos

        If we find a new product cert, we install it to /etc/pki/product
        and update the productid database to show what repo it came from.

        If we already have the product cert, but it now maps to a new or
        different repo, then update productid database with that info.

        It is possible for a single product cert to map to multiple repos.
        If multiple repo's all have the same product cert id metadata, we
        can get into this scenario. The anaconda install is an example of
        this, since the 'anaconda' repo the installer uses has the product
        cert metadata, but so does the corresponding rhel repo we get
        from autosubscribing. In those cases, we track both.

        Args:
            enabled: list of tuples of (product_cert, repo_id)
                     The repo's that are enabled=1 and have product
                     id metadata
            active: a set of names of repos that installed packages were
                    installed from.

        Returns:
             list of product certs that were installed

        Side Effects:
            can delete certs for some odd rhel5 scenarios, where we
            have to obsolete some deprecated certs
        """
        log.debug("Checking for product id certs to install or update.")
        products_to_install = []
        products_to_update_db = []
        products_installed = []

        log.debug("active %s", active)
        log.debug("enabled %s", enabled)
        # track updated product ids seperately in case we want
        # to run plugins
        products_to_update = []

        # enabled means we have a repo, it's enabled=1, it has a productid metadata
        # for it, and we understand that metadata. The cert for that productid
        # may or may not be installed at this point. If it is not installed,
        # we try to install it if there are packages installed from that repo

        # a ProductRepo object? ProductRepo's would have Repos and a
        # ProductCertificate
        #   .delete() -> delete ProductCert and it's entries in ProductDatabase
        #

        for cert, repo in enabled:
            log.debug("product cert: %s repo: %s" % (cert.products[0].id, repo))

            # nothing from this repo is installed
            #
            # check to see packages from this repo are installed and the repo
            # is considered active
            if repo not in active:
                continue

            # is this the same as v1 ProductCertificate.getProduct() ?
            # assume [0] indexed item is the same item
            p = cert.products[0]
            prod_hash = p.id

            # this is all workaround for some messed up certs in rhel5
            # are we installing workstation cert?
            if self._is_workstation(p):
                # is the desktop product cert installed?
                for pc in self.pdir.list():
                    if self._is_desktop(pc.products[0]):
                        log.info("removing obsolete desktop cert: %s" % pc.path)
                        # desktop product cert is installed,
                        # delete the desktop product cert
                        pc.delete()
                        self.pdir.refresh()  # must refresh to see the removal of the cert
                        self.db.delete(pc.products[0].id)
                        self.db.write()

            # if installing desktop cert, see if workstation exists on disk and skip
            # the write if so:
            if self._is_desktop(p):
                if self._workstation_cert_exists():
                    log.debug("skipping obsolete desktop cert")
                    continue

            # See if the product cert already exists
            #
            # if we don't find this product cert, install it
            # otherwise, update it if necessary
            #
            # ProductCert.is_installed() -> search pdir for ProductCert
            # ProductCert.install() -> add to install list
            if not self.pdir.find_by_product(prod_hash):
                products_to_install.append((p, cert))
            else:
                installed_product_cert = self.pdir.find_by_product(prod_hash)
                installed_product = installed_product_cert.products[0]
                # NOTE: this compares the Product in the ProductCert, but not
                # the ProductCert itself. We should probably compare the
                # ProductCert itself, which would start out as just comparing
                # it's contained Product with ComparableProduct
                cmp_product_cert = ComparableProductCert(cert)
                cmp_installed_product_cert = ComparableProductCert(installed_product_cert)
                if cmp_product_cert > cmp_installed_product_cert:
                    log.debug("Updating installed product cert for %s %s to %s %s" %
                            (installed_product.name, installed_product.version,
                             p.name, p.version))
                    products_to_update.append((p, cert))
                else:
                    log.debug("Latest version of product cert for %s %s is already installed, not updating" %
                            (p.name, p.version))

            # look up what repo's we know about for that prod id

            # ProductCertDb.install() could do this?
            # look up what repo's we know about for that prod has
            known_repos = self.db.find_repos(prod_hash)

            # ??? What happens for a installed product with no repo info, that
            # we think we should update?
            # known_repos is None means we have no repo info at all
            if known_repos is None or repo not in known_repos:
                products_to_update_db.append((p, repo))

        # see rhbz #977896
        # handle cases where we end up with workstation and desktop certs in
        # the same "transaction".

        # ProductCertDB.cleanup_workstation()
        # if we do this after setting up the lists, we dont need
        # to check per ProductRepo
        products_to_install = self._desktop_workstation_cleanup(products_to_install)
        products_to_update_db = self._desktop_workstation_cleanup(products_to_update_db)
        products_to_update = self._desktop_workstation_cleanup(products_to_update)

        db_updated = False
        for (product, repo) in products_to_update_db:
            # known_repos is None means we have no repo info at all
            log.info("Updating product db with %s -> %s" % (product.id, repo))
            # if we don't have a db entry for that prod->repo mapping, add one
            self.db.add(product.id, repo)
            db_updated = True

        if db_updated:
            self.db.write()

        products_installed = self.install_product_certs(products_to_install)
        products_updated = self.update_product_certs(products_to_update)

        #FIXME: nothing uses the return value here
        return (products_installed, products_updated)

    def install_product_certs(self, product_certs):
        # collect info, then do the needful later, so we can hook
        # up a plugin in between and let it munge these lists, so a plugin
        # could blacklist a product cert for example.
        self.plugin_manager.run('pre_product_id_install', product_list=product_certs)
        # ProductCertDb.install()
        #  -> for each ProductCert:
        #         ProductCert.install()
        #           - if that needs to update the db, do it
        #           - if db needs written, do it
        #   ProductCertDb.list_installed()
        products_installed = self.write_product_certs(product_certs)

        # FIXME: we should include productid db with the conduit here
        log.debug("about to run post_product_id_install")
        self.plugin_manager.run('post_product_id_install', product_list=products_installed)

        return products_installed

    def update_product_certs(self, product_certs):
        self.plugin_manager.run('pre_product_id_update', product_list=product_certs)

        products_updated = self.write_product_certs(product_certs)

        # FIXME: we should include productid db with the conduit here
        log.debug("about to run post_product_id_update")
        self.plugin_manager.run('post_product_id_update', product_list=products_updated)

        return products_updated

    def write_product_certs(self, product_certs):
        products_installed = []
        for (product, cert) in product_certs:
            fn = '%s.pem' % product.id
            path = self.pdir.abspath(fn)
            cert.write(path)
            self.pdir.refresh()
            log.info("Installed product cert %s: %s %s" % (product.id, product.name, cert.path))
            products_installed.append(cert)
        return products_installed

    def _workstation_cert_exists(self):
        for pc in self.pdir.list():
            if self._is_workstation(pc.products[0]):
                return True
        return False

    def _desktop_workstation_cleanup(self, product_cert_list):
        """Remove desktop product if desktop and workstations are marked for install/update"""
        if not self._list_has_workstation_and_desktop_cert(product_cert_list):
            # list doesnt have desktop and workstation, so do nothing
            return product_cert_list

        log.debug("Workstation and Desktop product certs found, removing Desktop cert from list to update")
        return [(product, cert) for (product, cert) in product_cert_list if not self._is_desktop(product)]

    def _list_has_workstation_and_desktop_cert(self, product_cert_list):
        """determine if product cert list has desktop and workstation certs"""
        has_workstation = False
        has_desktop = False
        for product, _product_cert in product_cert_list:
            if self._is_workstation(product):
                has_workstation = True
            if self._is_desktop(product):
                has_desktop = True
        return has_desktop and has_workstation

    # We should only delete productcerts if there are no
    # packages from that repo installed (not "active")
    # and we have the product cert installed.
    def update_removed(self, active, temp_disabled_repos=None):
        """remove product certs for inactive products

        For each installed product cert, check to see if we still have
        packages installed from the repo the product cert was installed
        from. If not, delete the product cert.

        With a few exceptions:
            1) if the productid db does not know about the product cert,
                do not delete it
            2) if the productid db doesn't know what repo that cert came
                from, do not delete it.
            3) If there were errors reading the repo metadata for any of
                the repos that provide that cert, do not delete it.
            4) If the product cert has providedtags for 'rhel*'

        Args:
            active: a set of repo name strings of the repos that installed
                    packages were installed from
        Side effects:
            deletes certs that need to be deleted
        """
        temp_disabled_repos = temp_disabled_repos or []
        certs_to_delete = []

        log.debug("Checking for product certs to remove. Active include: %s",
                  active)

        log.debug('Temporary disabled repos: %s' % temp_disabled_repos)

        disabled_repos = self.find_disabled_repos()

        for cert in self.pdir.list():
            product = cert.products[0]
            prod_hash = product.id

            # Protect all product certificates in /etc/pki/product-default
            # See: BZ: 1526622
            if cert.path.startswith(DEFAULT_PRODUCT_CERT_DIR):
                log.debug('Skipping prod. cert.: %s in protected directory' % cert.path)
                continue

            # FIXME: or if the productid.hs wasn't updated to reflect a new repo
            repos = self.db.find_repos(prod_hash)

            # If productid database does not know about the the product,
            # ie, repo is None (basically, return from a db.content.get(),
            # don't delete the cert because we don't know anything about it
            if repos is None or repos is []:
                # FIXME: this can also mean we need to update the product cert
                #        for prod_hash, since it is installed, but no longer maps to a repo
                # no repos to check, go to next cert
                log.debug('Skipping prod. cert.: %s without repos' % cert.path)
                continue

            delete_product_cert = True
            for repo in repos:
                # if we had errors with the repo or productid metadata
                # we could be very confused here, so do not
                # delete anything. see bz #736424
                if repo in self.meta_data_errors:
                    log.debug("%s has meta-data errors. Not deleting product cert %s.", repo, prod_hash)
                    delete_product_cert = False
                    continue

                # If product id maps to a repo that we know is disabled, don't delete it.
                if repo in disabled_repos and repo in active:
                    log.info("%s is disabled, but RPMs from this repo are installed. Not deleting product cert %s",
                             repo, prod_hash)
                    delete_product_cert = False
                    continue

                # do not delete a product cert if the repo[a] associated with it's prod_hash
                # has packages installed.
                if repo in active:
                    log.debug("%s is an active repo. Not deleting product cert %s", repo, prod_hash)
                    delete_product_cert = False
                    continue

                # If product id maps to a repo that we know is only temporarily
                # disabled, don't delete it.
                if repo in temp_disabled_repos:
                    log.warning("%s is disabled via yum cmdline. Not deleting product cert %s", repo, prod_hash)
                    delete_product_cert = False
                    continue

            # for this prod cert/hash, we know what repo[a] it's for, but nothing
            # appears to be installed from the repo[s]
            if delete_product_cert:
                certs_to_delete.append((product, cert))

        # TODO: plugin hook for pre_product_id_delete
        for product, cert in certs_to_delete:
            log.debug("None of the repos for %s are active: %s",
                     product.id,
                     self.db.find_repos(product.id))
            log.info("product cert %s for %s is being deleted" % (product.id, product.id))
            cert.delete()
            self.pdir.refresh()
            # TODO: plugin hook for post_product_id_delete

            # it should be safe to delete it's entry now, we either don't
            # know anything about it's repos, it doesnt have any, or none
            # of the repos are active
            self.db.delete(product.id)
            self.db.write()

    def _get_cert(self, filename):
        if filename.endswith('.gz'):
            f = GzipFile(filename)
        else:
            f = open(filename)
        try:
            pem = f.read()
            if type(pem) == bytes:
                pem = pem.decode('utf-8')
            cert = create_from_pem(pem)
            cert.pem = pem
            return cert
        finally:
            f.close()


if __name__ == '__main__':
    from subscription_manager.injectioninit import init_dep_injection
    init_dep_injection()

    logging.basicConfig(filename='/var/log/rhsm/rhsm.log', level=logging.DEBUG)
    log.debug('productid smoke testing')

    pm = ProductManager()
    pm.update_removed(active=set([]))



from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2010 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
#from gi.repository import Gdk

from subscription_manager.ga import Gdk as ga_Gdk
from subscription_manager.gui import widgets


class Progress(widgets.SubmanBaseWidget):

    widget_names = ['progressWindow', 'progressLabel', 'progressBar', 'statusLabel']
    gui_file = "progress"

    def __init__(self, title, label, support_markup=False):
        super(Progress, self).__init__()

        self.progressWindow.connect("delete-event", self._on_delete_event)
        cursor = ga_Gdk.Cursor(ga_Gdk.CursorType.WATCH)
        self.progressWindow.get_window().set_cursor(cursor)

        self.lastProgress = 0.0

        self.set_title(title)
        self.set_label(label, support_markup)

    def hide(self):
        self.progressWindow.hide()
        del self

    def set_title(self, text):
        self.progressWindow.set_title(text)

    def set_label(self, text, markup=False):
        if markup:
            self.progressLabel.set_markup(text)
        else:
            self.progressLabel.set_text(text)

    def pulse(self):
        """
        pulse for a glib mainloop timeout callback
        """
        # If it's been closed, don't attempt to pulse
        if self.progressBar:
            self.progressBar.pulse()
            return True
        return False

    def set_progress(self, amount, total):
        if total:
            i = min(1, amount / total)
        else:
            i = 1

        if i > self.lastProgress + .01 or i == 1:
            self.progressBar.set_fraction(i)
            if i == 1:
                # reset
                i = 0
            self.lastProgress = i

    def set_status_label(self, text):
        self.statusLabel.set_text(text)

    def set_transient_for(self, window):
        self.progressWindow.set_transient_for(window)

    def _on_delete_event(self, widget, event):
        # Block the progress bar from closing until we hide it
        return True
